defines = {
    "DT_ALPHA_NUM": 0x1,
    "DT_IA_64_NUM": 0x1,
    "DT_MIPS_NUM": 0x35,
    "DT_MIPS_RLD_VERSION": 0x70000001,
    "DT_MIPS_TIME_STAMP": 0x70000002,
    "DT_MIPS_ICHECKSUM": 0x70000003,
    "DT_MIPS_IVERSION": 0x70000004,
    "DT_MIPS_FLAGS": 0x70000005,
    "DT_MIPS_BASE_ADDRESS": 0x70000006,
    "DT_MIPS_MSYM": 0x70000007,
    "DT_MIPS_CONFLICT": 0x70000008,
    "DT_MIPS_LIBLIST": 0x70000009,
    "DT_MIPS_LOCAL_GOTNO": 0x7000000A,
    "DT_MIPS_CONFLICTNO": 0x7000000B,
    "DT_MIPS_LIBLISTNO": 0x70000010,
    "DT_MIPS_SYMTABNO": 0x70000011,
    "DT_MIPS_UNREFEXTNO": 0x70000012,
    "DT_MIPS_GOTSYM": 0x70000013,
    "DT_MIPS_HIPAGENO": 0x70000014,
    "DT_MIPS_RLD_MAP": 0x70000016,
    "DT_MIPS_DELTA_CLASS": 0x70000017,
    "DT_MIPS_DELTA_CLASS_NO": 0x70000018,
    "DT_MIPS_DELTA_INSTANCE": 0x70000019,
    "DT_MIPS_DELTA_INSTANCE_NO": 0x7000001A,
    "DT_MIPS_DELTA_RELOC": 0x7000001B,
    "DT_MIPS_DELTA_RELOC_NO": 0x7000001C,
    "DT_MIPS_DELTA_SYM": 0x7000001D,
    "DT_MIPS_DELTA_SYM_NO": 0x7000001E,
    "DT_MIPS_DELTA_CLASSSYM": 0x70000020,
    "DT_MIPS_DELTA_CLASSSYM_NO": 0x70000021,
    "DT_MIPS_CXX_FLAGS": 0x70000022,
    "DT_MIPS_PIXIE_INIT": 0x70000023,
    "DT_MIPS_SYMBOL_LIB": 0x70000024,
    "DT_MIPS_LOCALPAGE_GOTIDX": 0x70000025,
    "DT_MIPS_LOCAL_GOTIDX": 0x70000026,
    "DT_MIPS_HIDDEN_GOTIDX": 0x70000027,
    "DT_MIPS_PROTECTED_GOTIDX": 0x70000028,
    "DT_MIPS_OPTIONS": 0x70000029,
    "DT_MIPS_INTERFACE": 0x7000002A,
    "DT_MIPS_DYNSTR_ALIGN": 0x7000002B,
    "DT_MIPS_INTERFACE_SIZE": 0x7000002C,
    "DT_MIPS_RLD_TEXT_RESOLVE_ADDR": 0x7000002D,
    "DT_MIPS_PERF_SUFFIX": 0x7000002E,
    "DT_MIPS_COMPACT_SIZE": 0x7000002F,
    "DT_MIPS_GP_VALUE": 0x70000030,
    "DT_MIPS_AUX_DYNAMIC": 0x70000031,
    "DT_MIPS_PLTGOT": 0x70000032,
    "DT_MIPS_RWPLT": 0x70000034,
    "DT_NIOS2_GP": 0x70000002,
    "DT_PPC_NUM": 0x2,
    "DT_PPC64_NUM": 0x4,
    "DT_SPARC_NUM": 0x2,
    "DT_SPARC_REGISTER": 0x70000001,
    "EFA_PARISC_1_0": 0x20B,
    "EFA_PARISC_1_1": 0x210,
    "EFA_PARISC_2_0": 0x214,
    "EF_ALPHA_32BIT": 0x1,
    "EF_ALPHA_CANRELAX": 0x2,
    "EF_ARM_EABI_UNKNOWN": 0x0,
    "EF_ARM_RELEXEC": 0x1,
    "EF_ARM_HASENTRY": 0x2,
    "EF_ARM_SYMSARESORTED": 0x4,
    "EF_ARM_INTERWORK": 0x4,
    "EF_ARM_APCS_26": 0x8,
    "EF_ARM_DYNSYMSUSESEGIDX": 0x8,
    "EF_ARM_APCS_FLOAT": 0x10,
    "EF_ARM_MAPSYMSFIRST": 0x10,
    "EF_ARM_PIC": 0x20,
    "EF_ARM_ALIGN8": 0x40,
    "EF_ARM_NEW_ABI": 0x80,
    "EF_ARM_OLD_ABI": 0x100,
    "EF_ARM_SOFT_FLOAT": 0x200,
    "EF_ARM_ABI_FLOAT_SOFT": 0x200,
    "EF_ARM_ABI_FLOAT_HARD": 0x400,
    "EF_ARM_VFP_FLOAT": 0x400,
    "EF_ARM_MAVERICK_FLOAT": 0x800,
    "EF_ARM_LE8": 0x400000,
    "EF_ARM_BE8": 0x800000,
    "EF_ARM_EABI_VER1": 0x1000000,
    "EF_ARM_EABI_VER2": 0x2000000,
    "EF_ARM_EABI_VER3": 0x3000000,
    "EF_ARM_EABI_VER4": 0x4000000,
    "EF_ARM_EABI_VER5": 0x5000000,
    "EF_ARM_EABIMASK": 0xFF000000,
    "EF_IA_64_MASKOS": 0xF,
    "EF_IA_64_ABI64": 0x10,
    "EF_IA_64_ARCH": 0xFF000000,
    "EF_MIPS_ARCH_1": 0x0,
    "EF_MIPS_NOREORDER": 0x1,
    "EF_MIPS_PIC": 0x2,
    "EF_MIPS_CPIC": 0x4,
    "EF_MIPS_XGOT": 0x8,
    "EF_MIPS_64BIT_WHIRL": 0x10,
    "EF_MIPS_ABI2": 0x20,
    "EF_MIPS_ABI_ON32": 0x40,
    "EF_MIPS_FP64": 0x200,
    "EF_MIPS_NAN2008": 0x400,
    "EF_MIPS_ARCH_2": 0x10000000,
    "EF_MIPS_ARCH_3": 0x20000000,
    "EF_MIPS_ARCH_4": 0x30000000,
    "EF_MIPS_ARCH_5": 0x40000000,
    "EF_MIPS_ARCH_32": 0x50000000,
    "EF_MIPS_ARCH_64": 0x60000000,
    "EF_MIPS_ARCH_32R2": 0x70000000,
    "EF_MIPS_ARCH_64R2": 0x80000000,
    "EF_MIPS_ARCH": 0xF0000000,
    "EF_PARISC_ARCH": 0xFFFF,
    "EF_PARISC_TRAPNIL": 0x10000,
    "EF_PARISC_EXT": 0x20000,
    "EF_PARISC_LSB": 0x40000,
    "EF_PARISC_WIDE": 0x80000,
    "EF_PARISC_NO_KABP": 0x100000,
    "EF_PARISC_LAZYSWAP": 0x400000,
    "EF_PPC_RELOCATABLE_LIB": 0x8000,
    "EF_PPC_RELOCATABLE": 0x10000,
    "EF_PPC_EMB": 0x80000000,
    "EF_PPC64_ABI": 0x3,
    "EF_S390_HIGH_GPRS": 0x1,
    "EF_SH_UNKNOWN": 0x0,
    "EF_SH_DSP": 0x4,
    "EF_SH_MACH_MASK": 0x1F,
    "EF_SH1": 0x1,
    "EF_SH2": 0x2,
    "EF_SH2A": 0xD,
    "EF_SH2A_NOFPU": 0x13,
    "EF_SH2A_SH4_NOFPU": 0x15,
    "EF_SH2A_SH3_NOFPU": 0x16,
    "EF_SH2A_SH4": 0x17,
    "EF_SH2A_SH3E": 0x18,
    "EF_SH2E": 0xB,
    "EF_SH3": 0x3,
    "EF_SH3_DSP": 0x5,
    "EF_SH3_NOMMU": 0x14,
    "EF_SH3E": 0x8,
    "EF_SH4": 0x9,
    "EF_SH4_NOFPU": 0x10,
    "EF_SH4_NOMMU_NOFPU": 0x12,
    "EF_SH4A": 0xC,
    "EF_SH4A_NOFPU": 0x11,
    "EF_SH4AL_DSP": 0x6,
    "EF_SPARC_32PLUS": 0x100,
    "EF_SPARC_SUN_US1": 0x200,
    "EF_SPARC_HAL_R1": 0x400,
    "EF_SPARC_SUN_US3": 0x800,
    "EF_SPARC_LEDATA": 0x800000,
    "EF_SPARC_EXT_MASK": 0xFFFF00,
    "EF_SPARCV9_TSO": 0x0,
    "EF_SPARCV9_PSO": 0x1,
    "EF_SPARCV9_RMO": 0x2,
    "EF_SPARCV9_MM": 0x3,
    "LITUSE_ALPHA_ADDR": 0x0,
    "LITUSE_ALPHA_BASE": 0x1,
    "LITUSE_ALPHA_BYTOFF": 0x2,
    "LITUSE_ALPHA_JSR": 0x3,
    "LITUSE_ALPHA_TLS_GD": 0x4,
    "LITUSE_ALPHA_TLS_LDM": 0x5,
    "LL_DELAY_LOAD": 0x10,
    "LL_DELTA": 0x20,
    "LL_EXACT_MATCH": 0x1,
    "LL_EXPORTS": 0x8,
    "LL_IGNORE_INT_VER": 0x2,
    "LL_NONE": 0x0,
    "LL_REQUIRE_MINOR": 0x4,
    "MIPS_AFL_REG_NONE": 0x0,
    "MIPS_AFL_ASE_DSP": 0x1,
    "MIPS_AFL_FLAGS1_ODDSPREG": 0x1,
    "MIPS_AFL_REG_32": 0x1,
    "MIPS_AFL_EXT_XLR": 0x1,
    "MIPS_AFL_REG_64": 0x2,
    "MIPS_AFL_EXT_OCTEON2": 0x2,
    "MIPS_AFL_ASE_DSPR2": 0x2,
    "MIPS_AFL_REG_128": 0x3,
    "MIPS_AFL_EXT_OCTEONP": 0x3,
    "MIPS_AFL_ASE_EVA": 0x4,
    "MIPS_AFL_EXT_LOONGSON_3A": 0x4,
    "MIPS_AFL_EXT_OCTEON": 0x5,
    "MIPS_AFL_EXT_5900": 0x6,
    "MIPS_AFL_EXT_4650": 0x7,
    "MIPS_AFL_EXT_4010": 0x8,
    "MIPS_AFL_ASE_MCU": 0x8,
    "MIPS_AFL_EXT_4100": 0x9,
    "MIPS_AFL_EXT_3900": 0xA,
    "MIPS_AFL_EXT_10000": 0xB,
    "MIPS_AFL_EXT_SB1": 0xC,
    "MIPS_AFL_EXT_4111": 0xD,
    "MIPS_AFL_EXT_4120": 0xE,
    "MIPS_AFL_EXT_5400": 0xF,
    "MIPS_AFL_EXT_5500": 0x10,
    "MIPS_AFL_ASE_MDMX": 0x10,
    "MIPS_AFL_EXT_LOONGSON_2E": 0x11,
    "MIPS_AFL_EXT_LOONGSON_2F": 0x12,
    "MIPS_AFL_ASE_MIPS3D": 0x20,
    "MIPS_AFL_ASE_MT": 0x40,
    "MIPS_AFL_ASE_SMARTMIPS": 0x80,
    "MIPS_AFL_ASE_VIRT": 0x100,
    "MIPS_AFL_ASE_MSA": 0x200,
    "MIPS_AFL_ASE_MIPS16": 0x400,
    "MIPS_AFL_ASE_MICROMIPS": 0x800,
    "MIPS_AFL_ASE_XPA": 0x1000,
    "MIPS_AFL_ASE_MASK": 0x1FFF,
    "ODK_EXCEPTIONS": 0x2,
    "ODK_FILL": 0x5,
    "ODK_HWAND": 0x7,
    "ODK_HWOR": 0x8,
    "ODK_HWPATCH": 0x4,
    "ODK_NULL": 0x0,
    "ODK_PAD": 0x3,
    "ODK_REGINFO": 0x1,
    "ODK_TAGS": 0x6,
    "OEX_DISMISS": 0x80000,
    "OEX_FPDBUG": 0x40000,
    "OEX_FPU_INEX": 0x1,
    "OEX_FPU_UFLO": 0x2,
    "OEX_FPU_OFLO": 0x4,
    "OEX_FPU_DIV0": 0x8,
    "OEX_FPU_INVAL": 0x10,
    "OEX_FPU_MIN": 0x1F,
    "OEX_FPU_MAX": 0x1F00,
    "OEX_PAGE0": 0x10000,
    "OEX_SMM": 0x20000,
    "OHWA0_R4KEOP_CHECKED": 0x1,
    "OHWA1_R4KEOP_CLEAN": 0x2,
    "OHW_R4KEOP": 0x1,
    "OHW_R5KCVTL": 0x8,
    "OHW_R5KEOP": 0x4,
    "OHW_R8KPFETCH": 0x2,
    "OPAD_POSTFIX": 0x2,
    "OPAD_PREFIX": 0x1,
    "OPAD_SYMBOL": 0x4,
    "PF_ARM_SB": 0x10000000,
    "PF_ARM_PI": 0x20000000,
    "PF_ARM_ABS": 0x40000000,
    "PF_HP_PAGE_SIZE": 0x100000,
    "PF_HP_FAR_SHARED": 0x200000,
    "PF_HP_NEAR_SHARED": 0x400000,
    "PF_HP_CODE": 0x1000000,
    "PF_HP_MODIFY": 0x2000000,
    "PF_HP_LAZYSWAP": 0x4000000,
    "PF_HP_SBP": 0x8000000,
    "PF_IA_64_NORECOV": 0x80000000,
    "PF_MIPS_LOCAL": 0x10000000,
    "PF_PARISC_SBP": 0x8000000,
    "PPC64_OPT_TLS": 0x1,
    "PPC64_OPT_MULTI_TOC": 0x2,
    "PPC_OPT_TLS": 0x1,
    "PT_MIPS_REGINFO": 0x70000000,
    "PT_MIPS_RTPROC": 0x70000001,
    "PT_MIPS_OPTIONS": 0x70000002,
    "PT_MIPS_ABIFLAGS": 0x70000003,
    "PT_PARISC_ARCHEXT": 0x70000000,
    "PT_PARISC_UNWIND": 0x70000001,
    "RHF_CORD": 0x1000,
    "RHF_DEFAULT_DELAY_LOAD": 0x200,
    "RHF_DELTA_C_PLUS_PLUS": 0x40,
    "RHF_GUARANTEE_INIT": 0x20,
    "RHF_GUARANTEE_START_INIT": 0x80,
    "RHF_NO_LIBRARY_REPLACEMENT": 0x4,
    "RHF_NO_MOVE": 0x8,
    "RHF_NO_UNRES_UNDEF": 0x2000,
    "RHF_NONE": 0x0,
    "RHF_NOTPOT": 0x2,
    "RHF_PIXIE": 0x100,
    "RHF_QUICKSTART": 0x1,
    "RHF_REQUICKSTART": 0x400,
    "RHF_REQUICKSTARTED": 0x800,
    "RHF_RLD_ORDER_SAFE": 0x4000,
    "RHF_SGI_ONLY": 0x10,
    "R_386_NONE": 0x0,
    "R_386_32": 0x1,
    "R_386_PC32": 0x2,
    "R_386_GOT32": 0x3,
    "R_386_PLT32": 0x4,
    "R_386_COPY": 0x5,
    "R_386_GLOB_DAT": 0x6,
    "R_386_JMP_SLOT": 0x7,
    "R_386_RELATIVE": 0x8,
    "R_386_GOTOFF": 0x9,
    "R_386_GOTPC": 0xA,
    "R_386_32PLT": 0xB,
    "R_386_TLS_TPOFF": 0xE,
    "R_386_TLS_IE": 0xF,
    "R_386_TLS_GOTIE": 0x10,
    "R_386_TLS_LE": 0x11,
    "R_386_TLS_GD": 0x12,
    "R_386_TLS_LDM": 0x13,
    "R_386_16": 0x14,
    "R_386_PC16": 0x15,
    "R_386_8": 0x16,
    "R_386_PC8": 0x17,
    "R_386_TLS_GD_32": 0x18,
    "R_386_TLS_GD_PUSH": 0x19,
    "R_386_TLS_GD_CALL": 0x1A,
    "R_386_TLS_GD_POP": 0x1B,
    "R_386_TLS_LDM_32": 0x1C,
    "R_386_TLS_LDM_PUSH": 0x1D,
    "R_386_TLS_LDM_CALL": 0x1E,
    "R_386_TLS_LDM_POP": 0x1F,
    "R_386_TLS_LDO_32": 0x20,
    "R_386_TLS_IE_32": 0x21,
    "R_386_TLS_LE_32": 0x22,
    "R_386_TLS_DTPMOD32": 0x23,
    "R_386_TLS_DTPOFF32": 0x24,
    "R_386_TLS_TPOFF32": 0x25,
    "R_386_SIZE32": 0x26,
    "R_386_TLS_GOTDESC": 0x27,
    "R_386_TLS_DESC_CALL": 0x28,
    "R_386_TLS_DESC": 0x29,
    "R_386_IRELATIVE": 0x2A,
    "R_386_NUM": 0x2B,
    "R_390_NONE": 0x0,
    "R_390_8": 0x1,
    "R_390_12": 0x2,
    "R_390_16": 0x3,
    "R_390_32": 0x4,
    "R_390_PC32": 0x5,
    "R_390_GOT12": 0x6,
    "R_390_GOT32": 0x7,
    "R_390_PLT32": 0x8,
    "R_390_COPY": 0x9,
    "R_390_GLOB_DAT": 0xA,
    "R_390_JMP_SLOT": 0xB,
    "R_390_RELATIVE": 0xC,
    "R_390_GOTOFF32": 0xD,
    "R_390_GOTPC": 0xE,
    "R_390_GOT16": 0xF,
    "R_390_PC16": 0x10,
    "R_390_PC16DBL": 0x11,
    "R_390_PLT16DBL": 0x12,
    "R_390_PC32DBL": 0x13,
    "R_390_PLT32DBL": 0x14,
    "R_390_GOTPCDBL": 0x15,
    "R_390_64": 0x16,
    "R_390_PC64": 0x17,
    "R_390_GOT64": 0x18,
    "R_390_PLT64": 0x19,
    "R_390_GOTENT": 0x1A,
    "R_390_GOTOFF16": 0x1B,
    "R_390_GOTOFF64": 0x1C,
    "R_390_GOTPLT12": 0x1D,
    "R_390_GOTPLT16": 0x1E,
    "R_390_GOTPLT32": 0x1F,
    "R_390_GOTPLT64": 0x20,
    "R_390_GOTPLTENT": 0x21,
    "R_390_PLTOFF16": 0x22,
    "R_390_PLTOFF32": 0x23,
    "R_390_PLTOFF64": 0x24,
    "R_390_TLS_LOAD": 0x25,
    "R_390_TLS_GDCALL": 0x26,
    "R_390_TLS_LDCALL": 0x27,
    "R_390_TLS_GD32": 0x28,
    "R_390_TLS_GD64": 0x29,
    "R_390_TLS_GOTIE12": 0x2A,
    "R_390_TLS_GOTIE32": 0x2B,
    "R_390_TLS_GOTIE64": 0x2C,
    "R_390_TLS_LDM32": 0x2D,
    "R_390_TLS_LDM64": 0x2E,
    "R_390_TLS_IE32": 0x2F,
    "R_390_TLS_IE64": 0x30,
    "R_390_TLS_IEENT": 0x31,
    "R_390_TLS_LE32": 0x32,
    "R_390_TLS_LE64": 0x33,
    "R_390_TLS_LDO32": 0x34,
    "R_390_TLS_LDO64": 0x35,
    "R_390_TLS_DTPMOD": 0x36,
    "R_390_TLS_DTPOFF": 0x37,
    "R_390_TLS_TPOFF": 0x38,
    "R_390_20": 0x39,
    "R_390_GOT20": 0x3A,
    "R_390_GOTPLT20": 0x3B,
    "R_390_TLS_GOTIE20": 0x3C,
    "R_390_IRELATIVE": 0x3D,
    "R_390_NUM": 0x3E,
    "R_68K_NONE": 0x0,
    "R_68K_32": 0x1,
    "R_68K_16": 0x2,
    "R_68K_8": 0x3,
    "R_68K_PC32": 0x4,
    "R_68K_PC16": 0x5,
    "R_68K_PC8": 0x6,
    "R_68K_GOT32": 0x7,
    "R_68K_GOT16": 0x8,
    "R_68K_GOT8": 0x9,
    "R_68K_GOT32O": 0xA,
    "R_68K_GOT16O": 0xB,
    "R_68K_GOT8O": 0xC,
    "R_68K_PLT32": 0xD,
    "R_68K_PLT16": 0xE,
    "R_68K_PLT8": 0xF,
    "R_68K_PLT32O": 0x10,
    "R_68K_PLT16O": 0x11,
    "R_68K_PLT8O": 0x12,
    "R_68K_COPY": 0x13,
    "R_68K_GLOB_DAT": 0x14,
    "R_68K_JMP_SLOT": 0x15,
    "R_68K_RELATIVE": 0x16,
    "R_68K_TLS_GD32": 0x19,
    "R_68K_TLS_GD16": 0x1A,
    "R_68K_TLS_GD8": 0x1B,
    "R_68K_TLS_LDM32": 0x1C,
    "R_68K_TLS_LDM16": 0x1D,
    "R_68K_TLS_LDM8": 0x1E,
    "R_68K_TLS_LDO32": 0x1F,
    "R_68K_TLS_LDO16": 0x20,
    "R_68K_TLS_LDO8": 0x21,
    "R_68K_TLS_IE32": 0x22,
    "R_68K_TLS_IE16": 0x23,
    "R_68K_TLS_IE8": 0x24,
    "R_68K_TLS_LE32": 0x25,
    "R_68K_TLS_LE16": 0x26,
    "R_68K_TLS_LE8": 0x27,
    "R_68K_TLS_DTPMOD32": 0x28,
    "R_68K_TLS_DTPREL32": 0x29,
    "R_68K_TLS_TPREL32": 0x2A,
    "R_AARCH64_NONE": 0x0,
    "R_AARCH64_P32_ABS32": 0x1,
    "R_AARCH64_P32_COPY": 0xB4,
    "R_AARCH64_P32_GLOB_DAT": 0xB5,
    "R_AARCH64_P32_JUMP_SLOT": 0xB6,
    "R_AARCH64_P32_RELATIVE": 0xB7,
    "R_AARCH64_P32_TLS_DTPMOD": 0xB8,
    "R_AARCH64_P32_TLS_DTPREL": 0xB9,
    "R_AARCH64_P32_TLS_TPREL": 0xBA,
    "R_AARCH64_P32_TLSDESC": 0xBB,
    "R_AARCH64_P32_IRELATIVE": 0xBC,
    "R_AARCH64_ABS64": 0x101,
    "R_AARCH64_ABS32": 0x102,
    "R_AARCH64_ABS16": 0x103,
    "R_AARCH64_PREL64": 0x104,
    "R_AARCH64_PREL32": 0x105,
    "R_AARCH64_PREL16": 0x106,
    "R_AARCH64_MOVW_UABS_G0": 0x107,
    "R_AARCH64_MOVW_UABS_G0_NC": 0x108,
    "R_AARCH64_MOVW_UABS_G1": 0x109,
    "R_AARCH64_MOVW_UABS_G1_NC": 0x10A,
    "R_AARCH64_MOVW_UABS_G2": 0x10B,
    "R_AARCH64_MOVW_UABS_G2_NC": 0x10C,
    "R_AARCH64_MOVW_UABS_G3": 0x10D,
    "R_AARCH64_MOVW_SABS_G0": 0x10E,
    "R_AARCH64_MOVW_SABS_G1": 0x10F,
    "R_AARCH64_MOVW_SABS_G2": 0x110,
    "R_AARCH64_LD_PREL_LO19": 0x111,
    "R_AARCH64_ADR_PREL_LO21": 0x112,
    "R_AARCH64_ADR_PREL_PG_HI21": 0x113,
    "R_AARCH64_ADR_PREL_PG_HI21_NC": 0x114,
    "R_AARCH64_ADD_ABS_LO12_NC": 0x115,
    "R_AARCH64_LDST8_ABS_LO12_NC": 0x116,
    "R_AARCH64_TSTBR14": 0x117,
    "R_AARCH64_CONDBR19": 0x118,
    "R_AARCH64_JUMP26": 0x11A,
    "R_AARCH64_CALL26": 0x11B,
    "R_AARCH64_LDST16_ABS_LO12_NC": 0x11C,
    "R_AARCH64_LDST32_ABS_LO12_NC": 0x11D,
    "R_AARCH64_LDST64_ABS_LO12_NC": 0x11E,
    "R_AARCH64_MOVW_PREL_G0": 0x11F,
    "R_AARCH64_MOVW_PREL_G0_NC": 0x120,
    "R_AARCH64_MOVW_PREL_G1": 0x121,
    "R_AARCH64_MOVW_PREL_G1_NC": 0x122,
    "R_AARCH64_MOVW_PREL_G2": 0x123,
    "R_AARCH64_MOVW_PREL_G2_NC": 0x124,
    "R_AARCH64_MOVW_PREL_G3": 0x125,
    "R_AARCH64_LDST128_ABS_LO12_NC": 0x12B,
    "R_AARCH64_MOVW_GOTOFF_G0": 0x12C,
    "R_AARCH64_MOVW_GOTOFF_G0_NC": 0x12D,
    "R_AARCH64_MOVW_GOTOFF_G1": 0x12E,
    "R_AARCH64_MOVW_GOTOFF_G1_NC": 0x12F,
    "R_AARCH64_MOVW_GOTOFF_G2": 0x130,
    "R_AARCH64_MOVW_GOTOFF_G2_NC": 0x131,
    "R_AARCH64_MOVW_GOTOFF_G3": 0x132,
    "R_AARCH64_GOTREL64": 0x133,
    "R_AARCH64_GOTREL32": 0x134,
    "R_AARCH64_GOT_LD_PREL19": 0x135,
    "R_AARCH64_LD64_GOTOFF_LO15": 0x136,
    "R_AARCH64_ADR_GOT_PAGE": 0x137,
    "R_AARCH64_LD64_GOT_LO12_NC": 0x138,
    "R_AARCH64_LD64_GOTPAGE_LO15": 0x139,
    "R_AARCH64_TLSGD_ADR_PREL21": 0x200,
    "R_AARCH64_TLSGD_ADR_PAGE21": 0x201,
    "R_AARCH64_TLSGD_ADD_LO12_NC": 0x202,
    "R_AARCH64_TLSGD_MOVW_G1": 0x203,
    "R_AARCH64_TLSGD_MOVW_G0_NC": 0x204,
    "R_AARCH64_TLSLD_ADR_PREL21": 0x205,
    "R_AARCH64_TLSLD_ADR_PAGE21": 0x206,
    "R_AARCH64_TLSLD_ADD_LO12_NC": 0x207,
    "R_AARCH64_TLSLD_MOVW_G1": 0x208,
    "R_AARCH64_TLSLD_MOVW_G0_NC": 0x209,
    "R_AARCH64_TLSLD_LD_PREL19": 0x20A,
    "R_AARCH64_TLSLD_MOVW_DTPREL_G2": 0x20B,
    "R_AARCH64_TLSLD_MOVW_DTPREL_G1": 0x20C,
    "R_AARCH64_TLSLD_MOVW_DTPREL_G1_NC": 0x20D,
    "R_AARCH64_TLSLD_MOVW_DTPREL_G0": 0x20E,
    "R_AARCH64_TLSLD_MOVW_DTPREL_G0_NC": 0x20F,
    "R_AARCH64_TLSLD_ADD_DTPREL_HI12": 0x210,
    "R_AARCH64_TLSLD_ADD_DTPREL_LO12": 0x211,
    "R_AARCH64_TLSLD_ADD_DTPREL_LO12_NC": 0x212,
    "R_AARCH64_TLSLD_LDST8_DTPREL_LO12": 0x213,
    "R_AARCH64_TLSLD_LDST8_DTPREL_LO12_NC": 0x214,
    "R_AARCH64_TLSLD_LDST16_DTPREL_LO12": 0x215,
    "R_AARCH64_TLSLD_LDST16_DTPREL_LO12_NC": 0x216,
    "R_AARCH64_TLSLD_LDST32_DTPREL_LO12": 0x217,
    "R_AARCH64_TLSLD_LDST32_DTPREL_LO12_NC": 0x218,
    "R_AARCH64_TLSLD_LDST64_DTPREL_LO12": 0x219,
    "R_AARCH64_TLSLD_LDST64_DTPREL_LO12_NC": 0x21A,
    "R_AARCH64_TLSIE_MOVW_GOTTPREL_G1": 0x21B,
    "R_AARCH64_TLSIE_MOVW_GOTTPREL_G0_NC": 0x21C,
    "R_AARCH64_TLSIE_ADR_GOTTPREL_PAGE21": 0x21D,
    "R_AARCH64_TLSIE_LD64_GOTTPREL_LO12_NC": 0x21E,
    "R_AARCH64_TLSIE_LD_GOTTPREL_PREL19": 0x21F,
    "R_AARCH64_TLSLE_MOVW_TPREL_G2": 0x220,
    "R_AARCH64_TLSLE_MOVW_TPREL_G1": 0x221,
    "R_AARCH64_TLSLE_MOVW_TPREL_G1_NC": 0x222,
    "R_AARCH64_TLSLE_MOVW_TPREL_G0": 0x223,
    "R_AARCH64_TLSLE_MOVW_TPREL_G0_NC": 0x224,
    "R_AARCH64_TLSLE_ADD_TPREL_HI12": 0x225,
    "R_AARCH64_TLSLE_ADD_TPREL_LO12": 0x226,
    "R_AARCH64_TLSLE_ADD_TPREL_LO12_NC": 0x227,
    "R_AARCH64_TLSLE_LDST8_TPREL_LO12": 0x228,
    "R_AARCH64_TLSLE_LDST8_TPREL_LO12_NC": 0x229,
    "R_AARCH64_TLSLE_LDST16_TPREL_LO12": 0x22A,
    "R_AARCH64_TLSLE_LDST16_TPREL_LO12_NC": 0x22B,
    "R_AARCH64_TLSLE_LDST32_TPREL_LO12": 0x22C,
    "R_AARCH64_TLSLE_LDST32_TPREL_LO12_NC": 0x22D,
    "R_AARCH64_TLSLE_LDST64_TPREL_LO12": 0x22E,
    "R_AARCH64_TLSLE_LDST64_TPREL_LO12_NC": 0x22F,
    "R_AARCH64_TLSDESC_LD_PREL19": 0x230,
    "R_AARCH64_TLSDESC_ADR_PREL21": 0x231,
    "R_AARCH64_TLSDESC_ADR_PAGE21": 0x232,
    "R_AARCH64_TLSDESC_LD64_LO12": 0x233,
    "R_AARCH64_TLSDESC_ADD_LO12": 0x234,
    "R_AARCH64_TLSDESC_OFF_G1": 0x235,
    "R_AARCH64_TLSDESC_OFF_G0_NC": 0x236,
    "R_AARCH64_TLSDESC_LDR": 0x237,
    "R_AARCH64_TLSDESC_ADD": 0x238,
    "R_AARCH64_TLSDESC_CALL": 0x239,
    "R_AARCH64_TLSLE_LDST128_TPREL_LO12": 0x23A,
    "R_AARCH64_TLSLE_LDST128_TPREL_LO12_NC": 0x23B,
    "R_AARCH64_TLSLD_LDST128_DTPREL_LO12": 0x23C,
    "R_AARCH64_TLSLD_LDST128_DTPREL_LO12_NC": 0x23D,
    "R_AARCH64_COPY": 0x400,
    "R_AARCH64_GLOB_DAT": 0x401,
    "R_AARCH64_JUMP_SLOT": 0x402,
    "R_AARCH64_RELATIVE": 0x403,
    "R_AARCH64_TLS_DTPMOD": 0x404,
    "R_AARCH64_TLS_DTPREL": 0x405,
    "R_AARCH64_TLS_TPREL": 0x406,
    "R_AARCH64_TLSDESC": 0x407,
    "R_AARCH64_IRELATIVE": 0x408,
    "R_ALPHA_NONE": 0x0,
    "R_ALPHA_REFLONG": 0x1,
    "R_ALPHA_REFQUAD": 0x2,
    "R_ALPHA_GPREL32": 0x3,
    "R_ALPHA_LITERAL": 0x4,
    "R_ALPHA_LITUSE": 0x5,
    "R_ALPHA_GPDISP": 0x6,
    "R_ALPHA_BRADDR": 0x7,
    "R_ALPHA_HINT": 0x8,
    "R_ALPHA_SREL16": 0x9,
    "R_ALPHA_SREL32": 0xA,
    "R_ALPHA_SREL64": 0xB,
    "R_ALPHA_GPRELHIGH": 0x11,
    "R_ALPHA_GPRELLOW": 0x12,
    "R_ALPHA_GPREL16": 0x13,
    "R_ALPHA_COPY": 0x18,
    "R_ALPHA_GLOB_DAT": 0x19,
    "R_ALPHA_JMP_SLOT": 0x1A,
    "R_ALPHA_RELATIVE": 0x1B,
    "R_ALPHA_TLS_GD_HI": 0x1C,
    "R_ALPHA_TLSGD": 0x1D,
    "R_ALPHA_TLS_LDM": 0x1E,
    "R_ALPHA_DTPMOD64": 0x1F,
    "R_ALPHA_GOTDTPREL": 0x20,
    "R_ALPHA_DTPREL64": 0x21,
    "R_ALPHA_DTPRELHI": 0x22,
    "R_ALPHA_DTPRELLO": 0x23,
    "R_ALPHA_DTPREL16": 0x24,
    "R_ALPHA_GOTTPREL": 0x25,
    "R_ALPHA_TPREL64": 0x26,
    "R_ALPHA_TPRELHI": 0x27,
    "R_ALPHA_TPRELLO": 0x28,
    "R_ALPHA_TPREL16": 0x29,
    "R_ALPHA_NUM": 0x2E,
    "R_ARM_NONE": 0x0,
    "R_ARM_PC24": 0x1,
    "R_ARM_ABS32": 0x2,
    "R_ARM_REL32": 0x3,
    "R_ARM_PC13": 0x4,
    "R_ARM_ABS16": 0x5,
    "R_ARM_ABS12": 0x6,
    "R_ARM_THM_ABS5": 0x7,
    "R_ARM_ABS8": 0x8,
    "R_ARM_SBREL32": 0x9,
    "R_ARM_THM_PC22": 0xA,
    "R_ARM_THM_CALL": 0xA,
    "R_ARM_THM_PC8": 0xB,
    "R_ARM_AMP_VCALL9": 0xC,
    "R_ARM_TLS_DESC": 0xD,
    "R_ARM_SWI24": 0xD,
    "R_ARM_THM_SWI8": 0xE,
    "R_ARM_XPC25": 0xF,
    "R_ARM_THM_XPC22": 0x10,
    "R_ARM_TLS_DTPMOD32": 0x11,
    "R_ARM_TLS_DTPOFF32": 0x12,
    "R_ARM_TLS_TPOFF32": 0x13,
    "R_ARM_COPY": 0x14,
    "R_ARM_GLOB_DAT": 0x15,
    "R_ARM_JUMP_SLOT": 0x16,
    "R_ARM_RELATIVE": 0x17,
    "R_ARM_GOTOFF": 0x18,
    "R_ARM_GOTPC": 0x19,
    "R_ARM_GOT32": 0x1A,
    "R_ARM_PLT32": 0x1B,
    "R_ARM_CALL": 0x1C,
    "R_ARM_JUMP24": 0x1D,
    "R_ARM_THM_JUMP24": 0x1E,
    "R_ARM_BASE_ABS": 0x1F,
    "R_ARM_ALU_PCREL_7_0": 0x20,
    "R_ARM_ALU_PCREL_15_8": 0x21,
    "R_ARM_ALU_PCREL_23_15": 0x22,
    "R_ARM_LDR_SBREL_11_0": 0x23,
    "R_ARM_ALU_SBREL_19_12": 0x24,
    "R_ARM_ALU_SBREL_27_20": 0x25,
    "R_ARM_TARGET1": 0x26,
    "R_ARM_SBREL31": 0x27,
    "R_ARM_V4BX": 0x28,
    "R_ARM_TARGET2": 0x29,
    "R_ARM_PREL31": 0x2A,
    "R_ARM_MOVW_ABS_NC": 0x2B,
    "R_ARM_MOVT_ABS": 0x2C,
    "R_ARM_MOVW_PREL_NC": 0x2D,
    "R_ARM_MOVT_PREL": 0x2E,
    "R_ARM_THM_MOVW_ABS_NC": 0x2F,
    "R_ARM_THM_MOVT_ABS": 0x30,
    "R_ARM_THM_MOVW_PREL_NC": 0x31,
    "R_ARM_THM_MOVT_PREL": 0x32,
    "R_ARM_THM_JUMP19": 0x33,
    "R_ARM_THM_JUMP6": 0x34,
    "R_ARM_THM_ALU_PREL_11_0": 0x35,
    "R_ARM_THM_PC12": 0x36,
    "R_ARM_ABS32_NOI": 0x37,
    "R_ARM_REL32_NOI": 0x38,
    "R_ARM_ALU_PC_G0_NC": 0x39,
    "R_ARM_ALU_PC_G0": 0x3A,
    "R_ARM_ALU_PC_G1_NC": 0x3B,
    "R_ARM_ALU_PC_G1": 0x3C,
    "R_ARM_ALU_PC_G2": 0x3D,
    "R_ARM_LDR_PC_G1": 0x3E,
    "R_ARM_LDR_PC_G2": 0x3F,
    "R_ARM_LDRS_PC_G0": 0x40,
    "R_ARM_LDRS_PC_G1": 0x41,
    "R_ARM_LDRS_PC_G2": 0x42,
    "R_ARM_LDC_PC_G0": 0x43,
    "R_ARM_LDC_PC_G1": 0x44,
    "R_ARM_LDC_PC_G2": 0x45,
    "R_ARM_ALU_SB_G0_NC": 0x46,
    "R_ARM_ALU_SB_G0": 0x47,
    "R_ARM_ALU_SB_G1_NC": 0x48,
    "R_ARM_ALU_SB_G1": 0x49,
    "R_ARM_ALU_SB_G2": 0x4A,
    "R_ARM_LDR_SB_G0": 0x4B,
    "R_ARM_LDR_SB_G1": 0x4C,
    "R_ARM_LDR_SB_G2": 0x4D,
    "R_ARM_LDRS_SB_G0": 0x4E,
    "R_ARM_LDRS_SB_G1": 0x4F,
    "R_ARM_LDRS_SB_G2": 0x50,
    "R_ARM_LDC_SB_G0": 0x51,
    "R_ARM_LDC_SB_G1": 0x52,
    "R_ARM_LDC_SB_G2": 0x53,
    "R_ARM_MOVW_BREL_NC": 0x54,
    "R_ARM_MOVT_BREL": 0x55,
    "R_ARM_MOVW_BREL": 0x56,
    "R_ARM_THM_MOVW_BREL_NC": 0x57,
    "R_ARM_THM_MOVT_BREL": 0x58,
    "R_ARM_THM_MOVW_BREL": 0x59,
    "R_ARM_TLS_GOTDESC": 0x5A,
    "R_ARM_TLS_CALL": 0x5B,
    "R_ARM_TLS_DESCSEQ": 0x5C,
    "R_ARM_THM_TLS_CALL": 0x5D,
    "R_ARM_PLT32_ABS": 0x5E,
    "R_ARM_GOT_ABS": 0x5F,
    "R_ARM_GOT_PREL": 0x60,
    "R_ARM_GOT_BREL12": 0x61,
    "R_ARM_GOTOFF12": 0x62,
    "R_ARM_GOTRELAX": 0x63,
    "R_ARM_GNU_VTENTRY": 0x64,
    "R_ARM_GNU_VTINHERIT": 0x65,
    "R_ARM_THM_PC11": 0x66,
    "R_ARM_THM_PC9": 0x67,
    "R_ARM_TLS_GD32": 0x68,
    "R_ARM_TLS_LDM32": 0x69,
    "R_ARM_TLS_LDO32": 0x6A,
    "R_ARM_TLS_IE32": 0x6B,
    "R_ARM_TLS_LE32": 0x6C,
    "R_ARM_TLS_LDO12": 0x6D,
    "R_ARM_TLS_LE12": 0x6E,
    "R_ARM_TLS_IE12GP": 0x6F,
    "R_ARM_ME_TOO": 0x80,
    "R_ARM_THM_TLS_DESCSEQ": 0x81,
    "R_ARM_THM_TLS_DESCSEQ16": 0x81,
    "R_ARM_THM_TLS_DESCSEQ32": 0x82,
    "R_ARM_THM_GOT_BREL12": 0x83,
    "R_ARM_IRELATIVE": 0xA0,
    "R_ARM_RXPC25": 0xF9,
    "R_ARM_RSBREL32": 0xFA,
    "R_ARM_THM_RPC22": 0xFB,
    "R_ARM_RREL32": 0xFC,
    "R_ARM_RABS22": 0xFD,
    "R_ARM_RPC24": 0xFE,
    "R_ARM_RBASE": 0xFF,
    "R_ARM_NUM": 0x100,
    "R_CRIS_NONE": 0x0,
    "R_CRIS_8": 0x1,
    "R_CRIS_16": 0x2,
    "R_CRIS_32": 0x3,
    "R_CRIS_8_PCREL": 0x4,
    "R_CRIS_16_PCREL": 0x5,
    "R_CRIS_32_PCREL": 0x6,
    "R_CRIS_GNU_VTINHERIT": 0x7,
    "R_CRIS_GNU_VTENTRY": 0x8,
    "R_CRIS_COPY": 0x9,
    "R_CRIS_GLOB_DAT": 0xA,
    "R_CRIS_JUMP_SLOT": 0xB,
    "R_CRIS_RELATIVE": 0xC,
    "R_CRIS_16_GOT": 0xD,
    "R_CRIS_32_GOT": 0xE,
    "R_CRIS_16_GOTPLT": 0xF,
    "R_CRIS_32_GOTPLT": 0x10,
    "R_CRIS_32_GOTREL": 0x11,
    "R_CRIS_32_PLT_GOTREL": 0x12,
    "R_CRIS_32_PLT_PCREL": 0x13,
    "R_CRIS_NUM": 0x14,
    "R_IA64_NONE": 0x0,
    "R_IA64_IMM14": 0x21,
    "R_IA64_IMM22": 0x22,
    "R_IA64_IMM64": 0x23,
    "R_IA64_DIR32MSB": 0x24,
    "R_IA64_DIR32LSB": 0x25,
    "R_IA64_DIR64MSB": 0x26,
    "R_IA64_DIR64LSB": 0x27,
    "R_IA64_GPREL22": 0x2A,
    "R_IA64_GPREL64I": 0x2B,
    "R_IA64_GPREL32MSB": 0x2C,
    "R_IA64_GPREL32LSB": 0x2D,
    "R_IA64_GPREL64MSB": 0x2E,
    "R_IA64_GPREL64LSB": 0x2F,
    "R_IA64_LTOFF22": 0x32,
    "R_IA64_LTOFF64I": 0x33,
    "R_IA64_PLTOFF22": 0x3A,
    "R_IA64_PLTOFF64I": 0x3B,
    "R_IA64_PLTOFF64MSB": 0x3E,
    "R_IA64_PLTOFF64LSB": 0x3F,
    "R_IA64_FPTR64I": 0x43,
    "R_IA64_FPTR32MSB": 0x44,
    "R_IA64_FPTR32LSB": 0x45,
    "R_IA64_FPTR64MSB": 0x46,
    "R_IA64_FPTR64LSB": 0x47,
    "R_IA64_PCREL60B": 0x48,
    "R_IA64_PCREL21B": 0x49,
    "R_IA64_PCREL21M": 0x4A,
    "R_IA64_PCREL21F": 0x4B,
    "R_IA64_PCREL32MSB": 0x4C,
    "R_IA64_PCREL32LSB": 0x4D,
    "R_IA64_PCREL64MSB": 0x4E,
    "R_IA64_PCREL64LSB": 0x4F,
    "R_IA64_LTOFF_FPTR22": 0x52,
    "R_IA64_LTOFF_FPTR64I": 0x53,
    "R_IA64_LTOFF_FPTR32MSB": 0x54,
    "R_IA64_LTOFF_FPTR32LSB": 0x55,
    "R_IA64_LTOFF_FPTR64MSB": 0x56,
    "R_IA64_LTOFF_FPTR64LSB": 0x57,
    "R_IA64_SEGREL32MSB": 0x5C,
    "R_IA64_SEGREL32LSB": 0x5D,
    "R_IA64_SEGREL64MSB": 0x5E,
    "R_IA64_SEGREL64LSB": 0x5F,
    "R_IA64_SECREL32MSB": 0x64,
    "R_IA64_SECREL32LSB": 0x65,
    "R_IA64_SECREL64MSB": 0x66,
    "R_IA64_SECREL64LSB": 0x67,
    "R_IA64_REL32MSB": 0x6C,
    "R_IA64_REL32LSB": 0x6D,
    "R_IA64_REL64MSB": 0x6E,
    "R_IA64_REL64LSB": 0x6F,
    "R_IA64_LTV32MSB": 0x74,
    "R_IA64_LTV32LSB": 0x75,
    "R_IA64_LTV64MSB": 0x76,
    "R_IA64_LTV64LSB": 0x77,
    "R_IA64_PCREL21BI": 0x79,
    "R_IA64_PCREL22": 0x7A,
    "R_IA64_PCREL64I": 0x7B,
    "R_IA64_IPLTMSB": 0x80,
    "R_IA64_IPLTLSB": 0x81,
    "R_IA64_COPY": 0x84,
    "R_IA64_SUB": 0x85,
    "R_IA64_LTOFF22X": 0x86,
    "R_IA64_LDXMOV": 0x87,
    "R_IA64_TPREL14": 0x91,
    "R_IA64_TPREL22": 0x92,
    "R_IA64_TPREL64I": 0x93,
    "R_IA64_TPREL64MSB": 0x96,
    "R_IA64_TPREL64LSB": 0x97,
    "R_IA64_LTOFF_TPREL22": 0x9A,
    "R_IA64_DTPMOD64MSB": 0xA6,
    "R_IA64_DTPMOD64LSB": 0xA7,
    "R_IA64_LTOFF_DTPMOD22": 0xAA,
    "R_IA64_DTPREL14": 0xB1,
    "R_IA64_DTPREL22": 0xB2,
    "R_IA64_DTPREL64I": 0xB3,
    "R_IA64_DTPREL32MSB": 0xB4,
    "R_IA64_DTPREL32LSB": 0xB5,
    "R_IA64_DTPREL64MSB": 0xB6,
    "R_IA64_DTPREL64LSB": 0xB7,
    "R_IA64_LTOFF_DTPREL22": 0xBA,
    "R_M32R_NONE": 0x0,
    "R_M32R_16": 0x1,
    "R_M32R_32": 0x2,
    "R_M32R_24": 0x3,
    "R_M32R_10_PCREL": 0x4,
    "R_M32R_18_PCREL": 0x5,
    "R_M32R_26_PCREL": 0x6,
    "R_M32R_HI16_ULO": 0x7,
    "R_M32R_HI16_SLO": 0x8,
    "R_M32R_LO16": 0x9,
    "R_M32R_SDA16": 0xA,
    "R_M32R_GNU_VTINHERIT": 0xB,
    "R_M32R_GNU_VTENTRY": 0xC,
    "R_M32R_16_RELA": 0x21,
    "R_M32R_32_RELA": 0x22,
    "R_M32R_24_RELA": 0x23,
    "R_M32R_10_PCREL_RELA": 0x24,
    "R_M32R_18_PCREL_RELA": 0x25,
    "R_M32R_26_PCREL_RELA": 0x26,
    "R_M32R_HI16_ULO_RELA": 0x27,
    "R_M32R_HI16_SLO_RELA": 0x28,
    "R_M32R_LO16_RELA": 0x29,
    "R_M32R_SDA16_RELA": 0x2A,
    "R_M32R_RELA_GNU_VTINHERIT": 0x2B,
    "R_M32R_RELA_GNU_VTENTRY": 0x2C,
    "R_M32R_REL32": 0x2D,
    "R_M32R_GOT24": 0x30,
    "R_M32R_26_PLTREL": 0x31,
    "R_M32R_COPY": 0x32,
    "R_M32R_GLOB_DAT": 0x33,
    "R_M32R_JMP_SLOT": 0x34,
    "R_M32R_RELATIVE": 0x35,
    "R_M32R_GOTOFF": 0x36,
    "R_M32R_GOTPC24": 0x37,
    "R_M32R_GOT16_HI_ULO": 0x38,
    "R_M32R_GOT16_HI_SLO": 0x39,
    "R_M32R_GOT16_LO": 0x3A,
    "R_M32R_GOTPC_HI_ULO": 0x3B,
    "R_M32R_GOTPC_HI_SLO": 0x3C,
    "R_M32R_GOTPC_LO": 0x3D,
    "R_M32R_GOTOFF_HI_ULO": 0x3E,
    "R_M32R_GOTOFF_HI_SLO": 0x3F,
    "R_M32R_GOTOFF_LO": 0x40,
    "R_M32R_NUM": 0x100,
    "R_MICROBLAZE_NONE": 0x0,
    "R_MICROBLAZE_32": 0x1,
    "R_MICROBLAZE_32_PCREL": 0x2,
    "R_MICROBLAZE_64_PCREL": 0x3,
    "R_MICROBLAZE_32_PCREL_LO": 0x4,
    "R_MICROBLAZE_64": 0x5,
    "R_MICROBLAZE_32_LO": 0x6,
    "R_MICROBLAZE_SRO32": 0x7,
    "R_MICROBLAZE_SRW32": 0x8,
    "R_MICROBLAZE_64_NONE": 0x9,
    "R_MICROBLAZE_32_SYM_OP_SYM": 0xA,
    "R_MICROBLAZE_GNU_VTINHERIT": 0xB,
    "R_MICROBLAZE_GNU_VTENTRY": 0xC,
    "R_MICROBLAZE_GOTPC_64": 0xD,
    "R_MICROBLAZE_GOT_64": 0xE,
    "R_MICROBLAZE_PLT_64": 0xF,
    "R_MICROBLAZE_REL": 0x10,
    "R_MICROBLAZE_JUMP_SLOT": 0x11,
    "R_MICROBLAZE_GLOB_DAT": 0x12,
    "R_MICROBLAZE_GOTOFF_64": 0x13,
    "R_MICROBLAZE_GOTOFF_32": 0x14,
    "R_MICROBLAZE_COPY": 0x15,
    "R_MICROBLAZE_TLS": 0x16,
    "R_MICROBLAZE_TLSGD": 0x17,
    "R_MICROBLAZE_TLSLD": 0x18,
    "R_MICROBLAZE_TLSDTPMOD32": 0x19,
    "R_MICROBLAZE_TLSDTPREL32": 0x1A,
    "R_MICROBLAZE_TLSDTPREL64": 0x1B,
    "R_MICROBLAZE_TLSGOTTPREL32": 0x1C,
    "R_MICROBLAZE_TLSTPREL32": 0x1D,
    "R_MIPS_NONE": 0x0,
    "R_MIPS_16": 0x1,
    "R_MIPS_32": 0x2,
    "R_MIPS_REL32": 0x3,
    "R_MIPS_26": 0x4,
    "R_MIPS_HI16": 0x5,
    "R_MIPS_LO16": 0x6,
    "R_MIPS_GPREL16": 0x7,
    "R_MIPS_LITERAL": 0x8,
    "R_MIPS_GOT16": 0x9,
    "R_MIPS_PC16": 0xA,
    "R_MIPS_CALL16": 0xB,
    "R_MIPS_GPREL32": 0xC,
    "R_MIPS_SHIFT5": 0x10,
    "R_MIPS_SHIFT6": 0x11,
    "R_MIPS_64": 0x12,
    "R_MIPS_GOT_DISP": 0x13,
    "R_MIPS_GOT_PAGE": 0x14,
    "R_MIPS_GOT_OFST": 0x15,
    "R_MIPS_GOT_HI16": 0x16,
    "R_MIPS_GOT_LO16": 0x17,
    "R_MIPS_SUB": 0x18,
    "R_MIPS_INSERT_A": 0x19,
    "R_MIPS_INSERT_B": 0x1A,
    "R_MIPS_DELETE": 0x1B,
    "R_MIPS_HIGHER": 0x1C,
    "R_MIPS_HIGHEST": 0x1D,
    "R_MIPS_CALL_HI16": 0x1E,
    "R_MIPS_CALL_LO16": 0x1F,
    "R_MIPS_SCN_DISP": 0x20,
    "R_MIPS_REL16": 0x21,
    "R_MIPS_ADD_IMMEDIATE": 0x22,
    "R_MIPS_PJUMP": 0x23,
    "R_MIPS_RELGOT": 0x24,
    "R_MIPS_JALR": 0x25,
    "R_MIPS_TLS_DTPMOD32": 0x26,
    "R_MIPS_TLS_DTPREL32": 0x27,
    "R_MIPS_TLS_DTPMOD64": 0x28,
    "R_MIPS_TLS_DTPREL64": 0x29,
    "R_MIPS_TLS_GD": 0x2A,
    "R_MIPS_TLS_LDM": 0x2B,
    "R_MIPS_TLS_DTPREL_HI16": 0x2C,
    "R_MIPS_TLS_DTPREL_LO16": 0x2D,
    "R_MIPS_TLS_GOTTPREL": 0x2E,
    "R_MIPS_TLS_TPREL32": 0x2F,
    "R_MIPS_TLS_TPREL64": 0x30,
    "R_MIPS_TLS_TPREL_HI16": 0x31,
    "R_MIPS_TLS_TPREL_LO16": 0x32,
    "R_MIPS_GLOB_DAT": 0x33,
    "R_MIPS_COPY": 0x7E,
    "R_MIPS_JUMP_SLOT": 0x7F,
    "R_MIPS_NUM": 0x80,
    "R_MN10300_NONE": 0x0,
    "R_MN10300_32": 0x1,
    "R_MN10300_16": 0x2,
    "R_MN10300_8": 0x3,
    "R_MN10300_PCREL32": 0x4,
    "R_MN10300_PCREL16": 0x5,
    "R_MN10300_PCREL8": 0x6,
    "R_MN10300_GNU_VTINHERIT": 0x7,
    "R_MN10300_GNU_VTENTRY": 0x8,
    "R_MN10300_24": 0x9,
    "R_MN10300_GOTPC32": 0xA,
    "R_MN10300_GOTPC16": 0xB,
    "R_MN10300_GOTOFF32": 0xC,
    "R_MN10300_GOTOFF24": 0xD,
    "R_MN10300_GOTOFF16": 0xE,
    "R_MN10300_PLT32": 0xF,
    "R_MN10300_PLT16": 0x10,
    "R_MN10300_GOT32": 0x11,
    "R_MN10300_GOT24": 0x12,
    "R_MN10300_GOT16": 0x13,
    "R_MN10300_COPY": 0x14,
    "R_MN10300_GLOB_DAT": 0x15,
    "R_MN10300_JMP_SLOT": 0x16,
    "R_MN10300_RELATIVE": 0x17,
    "R_MN10300_TLS_GD": 0x18,
    "R_MN10300_TLS_LD": 0x19,
    "R_MN10300_TLS_LDO": 0x1A,
    "R_MN10300_TLS_GOTIE": 0x1B,
    "R_MN10300_TLS_IE": 0x1C,
    "R_MN10300_TLS_LE": 0x1D,
    "R_MN10300_TLS_DTPMOD": 0x1E,
    "R_MN10300_TLS_DTPOFF": 0x1F,
    "R_MN10300_TLS_TPOFF": 0x20,
    "R_MN10300_SYM_DIFF": 0x21,
    "R_MN10300_ALIGN": 0x22,
    "R_MN10300_NUM": 0x23,
    "R_NIOS2_NONE": 0x0,
    "R_NIOS2_S16": 0x1,
    "R_NIOS2_U16": 0x2,
    "R_NIOS2_PCREL16": 0x3,
    "R_NIOS2_CALL26": 0x4,
    "R_NIOS2_IMM5": 0x5,
    "R_NIOS2_CACHE_OPX": 0x6,
    "R_NIOS2_IMM6": 0x7,
    "R_NIOS2_IMM8": 0x8,
    "R_NIOS2_HI16": 0x9,
    "R_NIOS2_LO16": 0xA,
    "R_NIOS2_HIADJ16": 0xB,
    "R_NIOS2_BFD_RELOC_32": 0xC,
    "R_NIOS2_BFD_RELOC_16": 0xD,
    "R_NIOS2_BFD_RELOC_8": 0xE,
    "R_NIOS2_GPREL": 0xF,
    "R_NIOS2_GNU_VTINHERIT": 0x10,
    "R_NIOS2_GNU_VTENTRY": 0x11,
    "R_NIOS2_UJMP": 0x12,
    "R_NIOS2_CJMP": 0x13,
    "R_NIOS2_CALLR": 0x14,
    "R_NIOS2_ALIGN": 0x15,
    "R_NIOS2_GOT16": 0x16,
    "R_NIOS2_CALL16": 0x17,
    "R_NIOS2_GOTOFF_LO": 0x18,
    "R_NIOS2_GOTOFF_HA": 0x19,
    "R_NIOS2_PCREL_LO": 0x1A,
    "R_NIOS2_PCREL_HA": 0x1B,
    "R_NIOS2_TLS_GD16": 0x1C,
    "R_NIOS2_TLS_LDM16": 0x1D,
    "R_NIOS2_TLS_LDO16": 0x1E,
    "R_NIOS2_TLS_IE16": 0x1F,
    "R_NIOS2_TLS_LE16": 0x20,
    "R_NIOS2_TLS_DTPMOD": 0x21,
    "R_NIOS2_TLS_DTPREL": 0x22,
    "R_NIOS2_TLS_TPREL": 0x23,
    "R_NIOS2_COPY": 0x24,
    "R_NIOS2_GLOB_DAT": 0x25,
    "R_NIOS2_JUMP_SLOT": 0x26,
    "R_NIOS2_RELATIVE": 0x27,
    "R_NIOS2_GOTOFF": 0x28,
    "R_NIOS2_CALL26_NOAT": 0x29,
    "R_NIOS2_GOT_LO": 0x2A,
    "R_NIOS2_GOT_HA": 0x2B,
    "R_NIOS2_CALL_LO": 0x2C,
    "R_NIOS2_CALL_HA": 0x2D,
    "R_PARISC_NONE": 0x0,
    "R_PARISC_DIR32": 0x1,
    "R_PARISC_DIR21L": 0x2,
    "R_PARISC_DIR17R": 0x3,
    "R_PARISC_DIR17F": 0x4,
    "R_PARISC_DIR14R": 0x6,
    "R_PARISC_PCREL32": 0x9,
    "R_PARISC_PCREL21L": 0xA,
    "R_PARISC_PCREL17R": 0xB,
    "R_PARISC_PCREL17F": 0xC,
    "R_PARISC_PCREL14R": 0xE,
    "R_PARISC_DPREL21L": 0x12,
    "R_PARISC_DPREL14R": 0x16,
    "R_PARISC_GPREL21L": 0x1A,
    "R_PARISC_GPREL14R": 0x1E,
    "R_PARISC_LTOFF21L": 0x22,
    "R_PARISC_LTOFF14R": 0x26,
    "R_PARISC_SECREL32": 0x29,
    "R_PARISC_SEGBASE": 0x30,
    "R_PARISC_SEGREL32": 0x31,
    "R_PARISC_PLTOFF21L": 0x32,
    "R_PARISC_PLTOFF14R": 0x36,
    "R_PARISC_LTOFF_FPTR32": 0x39,
    "R_PARISC_LTOFF_FPTR21L": 0x3A,
    "R_PARISC_LTOFF_FPTR14R": 0x3E,
    "R_PARISC_FPTR64": 0x40,
    "R_PARISC_PLABEL32": 0x41,
    "R_PARISC_PLABEL21L": 0x42,
    "R_PARISC_PLABEL14R": 0x46,
    "R_PARISC_PCREL64": 0x48,
    "R_PARISC_PCREL22F": 0x4A,
    "R_PARISC_PCREL14WR": 0x4B,
    "R_PARISC_PCREL14DR": 0x4C,
    "R_PARISC_PCREL16F": 0x4D,
    "R_PARISC_PCREL16WF": 0x4E,
    "R_PARISC_PCREL16DF": 0x4F,
    "R_PARISC_DIR64": 0x50,
    "R_PARISC_DIR14WR": 0x53,
    "R_PARISC_DIR14DR": 0x54,
    "R_PARISC_DIR16F": 0x55,
    "R_PARISC_DIR16WF": 0x56,
    "R_PARISC_DIR16DF": 0x57,
    "R_PARISC_GPREL64": 0x58,
    "R_PARISC_GPREL14WR": 0x5B,
    "R_PARISC_GPREL14DR": 0x5C,
    "R_PARISC_GPREL16F": 0x5D,
    "R_PARISC_GPREL16WF": 0x5E,
    "R_PARISC_GPREL16DF": 0x5F,
    "R_PARISC_LTOFF64": 0x60,
    "R_PARISC_LTOFF14WR": 0x63,
    "R_PARISC_LTOFF14DR": 0x64,
    "R_PARISC_LTOFF16F": 0x65,
    "R_PARISC_LTOFF16WF": 0x66,
    "R_PARISC_LTOFF16DF": 0x67,
    "R_PARISC_SECREL64": 0x68,
    "R_PARISC_SEGREL64": 0x70,
    "R_PARISC_PLTOFF14WR": 0x73,
    "R_PARISC_PLTOFF14DR": 0x74,
    "R_PARISC_PLTOFF16F": 0x75,
    "R_PARISC_PLTOFF16WF": 0x76,
    "R_PARISC_PLTOFF16DF": 0x77,
    "R_PARISC_LTOFF_FPTR64": 0x78,
    "R_PARISC_LTOFF_FPTR14WR": 0x7B,
    "R_PARISC_LTOFF_FPTR14DR": 0x7C,
    "R_PARISC_LTOFF_FPTR16F": 0x7D,
    "R_PARISC_LTOFF_FPTR16WF": 0x7E,
    "R_PARISC_LTOFF_FPTR16DF": 0x7F,
    "R_PARISC_LORESERVE": 0x80,
    "R_PARISC_COPY": 0x80,
    "R_PARISC_IPLT": 0x81,
    "R_PARISC_EPLT": 0x82,
    "R_PARISC_TPREL32": 0x99,
    "R_PARISC_TPREL21L": 0x9A,
    "R_PARISC_TPREL14R": 0x9E,
    "R_PARISC_LTOFF_TP21L": 0xA2,
    "R_PARISC_LTOFF_TP14R": 0xA6,
    "R_PARISC_LTOFF_TP14F": 0xA7,
    "R_PARISC_TPREL64": 0xD8,
    "R_PARISC_TPREL14WR": 0xDB,
    "R_PARISC_TPREL14DR": 0xDC,
    "R_PARISC_TPREL16F": 0xDD,
    "R_PARISC_TPREL16WF": 0xDE,
    "R_PARISC_TPREL16DF": 0xDF,
    "R_PARISC_LTOFF_TP64": 0xE0,
    "R_PARISC_LTOFF_TP14WR": 0xE3,
    "R_PARISC_LTOFF_TP14DR": 0xE4,
    "R_PARISC_LTOFF_TP16F": 0xE5,
    "R_PARISC_LTOFF_TP16WF": 0xE6,
    "R_PARISC_LTOFF_TP16DF": 0xE7,
    "R_PARISC_GNU_VTENTRY": 0xE8,
    "R_PARISC_GNU_VTINHERIT": 0xE9,
    "R_PARISC_TLS_GD21L": 0xEA,
    "R_PARISC_TLS_GD14R": 0xEB,
    "R_PARISC_TLS_GDCALL": 0xEC,
    "R_PARISC_TLS_LDM21L": 0xED,
    "R_PARISC_TLS_LDM14R": 0xEE,
    "R_PARISC_TLS_LDMCALL": 0xEF,
    "R_PARISC_TLS_LDO21L": 0xF0,
    "R_PARISC_TLS_LDO14R": 0xF1,
    "R_PARISC_TLS_DTPMOD32": 0xF2,
    "R_PARISC_TLS_DTPMOD64": 0xF3,
    "R_PARISC_TLS_DTPOFF32": 0xF4,
    "R_PARISC_TLS_DTPOFF64": 0xF5,
    "R_PARISC_HIRESERVE": 0xFF,
    "R_PPC_NONE": 0x0,
    "R_PPC_ADDR32": 0x1,
    "R_PPC_ADDR24": 0x2,
    "R_PPC_ADDR16": 0x3,
    "R_PPC_ADDR16_LO": 0x4,
    "R_PPC_ADDR16_HI": 0x5,
    "R_PPC_ADDR16_HA": 0x6,
    "R_PPC_ADDR14": 0x7,
    "R_PPC_ADDR14_BRTAKEN": 0x8,
    "R_PPC_ADDR14_BRNTAKEN": 0x9,
    "R_PPC_REL24": 0xA,
    "R_PPC_REL14": 0xB,
    "R_PPC_REL14_BRTAKEN": 0xC,
    "R_PPC_REL14_BRNTAKEN": 0xD,
    "R_PPC_GOT16": 0xE,
    "R_PPC_GOT16_LO": 0xF,
    "R_PPC_GOT16_HI": 0x10,
    "R_PPC_GOT16_HA": 0x11,
    "R_PPC_PLTREL24": 0x12,
    "R_PPC_COPY": 0x13,
    "R_PPC_GLOB_DAT": 0x14,
    "R_PPC_JMP_SLOT": 0x15,
    "R_PPC_RELATIVE": 0x16,
    "R_PPC_LOCAL24PC": 0x17,
    "R_PPC_UADDR32": 0x18,
    "R_PPC_UADDR16": 0x19,
    "R_PPC_REL32": 0x1A,
    "R_PPC_PLT32": 0x1B,
    "R_PPC_PLTREL32": 0x1C,
    "R_PPC_PLT16_LO": 0x1D,
    "R_PPC_PLT16_HI": 0x1E,
    "R_PPC_PLT16_HA": 0x1F,
    "R_PPC_SDAREL16": 0x20,
    "R_PPC_SECTOFF": 0x21,
    "R_PPC_SECTOFF_LO": 0x22,
    "R_PPC_SECTOFF_HI": 0x23,
    "R_PPC_SECTOFF_HA": 0x24,
    "R_PPC_TLS": 0x43,
    "R_PPC_DTPMOD32": 0x44,
    "R_PPC_TPREL16": 0x45,
    "R_PPC_TPREL16_LO": 0x46,
    "R_PPC_TPREL16_HI": 0x47,
    "R_PPC_TPREL16_HA": 0x48,
    "R_PPC_TPREL32": 0x49,
    "R_PPC_DTPREL16": 0x4A,
    "R_PPC_DTPREL16_LO": 0x4B,
    "R_PPC_DTPREL16_HI": 0x4C,
    "R_PPC_DTPREL16_HA": 0x4D,
    "R_PPC_DTPREL32": 0x4E,
    "R_PPC_GOT_TLSGD16": 0x4F,
    "R_PPC_GOT_TLSGD16_LO": 0x50,
    "R_PPC_GOT_TLSGD16_HI": 0x51,
    "R_PPC_GOT_TLSGD16_HA": 0x52,
    "R_PPC_GOT_TLSLD16": 0x53,
    "R_PPC_GOT_TLSLD16_LO": 0x54,
    "R_PPC_GOT_TLSLD16_HI": 0x55,
    "R_PPC_GOT_TLSLD16_HA": 0x56,
    "R_PPC_GOT_TPREL16": 0x57,
    "R_PPC_GOT_TPREL16_LO": 0x58,
    "R_PPC_GOT_TPREL16_HI": 0x59,
    "R_PPC_GOT_TPREL16_HA": 0x5A,
    "R_PPC_GOT_DTPREL16": 0x5B,
    "R_PPC_GOT_DTPREL16_LO": 0x5C,
    "R_PPC_GOT_DTPREL16_HI": 0x5D,
    "R_PPC_GOT_DTPREL16_HA": 0x5E,
    "R_PPC_TLSGD": 0x5F,
    "R_PPC_TLSLD": 0x60,
    "R_PPC_EMB_NADDR32": 0x65,
    "R_PPC_EMB_NADDR16": 0x66,
    "R_PPC_EMB_NADDR16_LO": 0x67,
    "R_PPC_EMB_NADDR16_HI": 0x68,
    "R_PPC_EMB_NADDR16_HA": 0x69,
    "R_PPC_EMB_SDAI16": 0x6A,
    "R_PPC_EMB_SDA2I16": 0x6B,
    "R_PPC_EMB_SDA2REL": 0x6C,
    "R_PPC_EMB_SDA21": 0x6D,
    "R_PPC_EMB_MRKREF": 0x6E,
    "R_PPC_EMB_RELSEC16": 0x6F,
    "R_PPC_EMB_RELST_LO": 0x70,
    "R_PPC_EMB_RELST_HI": 0x71,
    "R_PPC_EMB_RELST_HA": 0x72,
    "R_PPC_EMB_BIT_FLD": 0x73,
    "R_PPC_EMB_RELSDA": 0x74,
    "R_PPC_DIAB_SDA21_LO": 0xB4,
    "R_PPC_DIAB_SDA21_HI": 0xB5,
    "R_PPC_DIAB_SDA21_HA": 0xB6,
    "R_PPC_DIAB_RELSDA_LO": 0xB7,
    "R_PPC_DIAB_RELSDA_HI": 0xB8,
    "R_PPC_DIAB_RELSDA_HA": 0xB9,
    "R_PPC_IRELATIVE": 0xF8,
    "R_PPC_REL16": 0xF9,
    "R_PPC_REL16_LO": 0xFA,
    "R_PPC_REL16_HI": 0xFB,
    "R_PPC_REL16_HA": 0xFC,
    "R_PPC_TOC16": 0xFF,
    "R_PPC64_NONE": 0x0,
    "R_PPC64_ADDR32": 0x1,
    "R_PPC64_ADDR24": 0x2,
    "R_PPC64_ADDR16": 0x3,
    "R_PPC64_ADDR16_LO": 0x4,
    "R_PPC64_ADDR16_HI": 0x5,
    "R_PPC64_ADDR16_HA": 0x6,
    "R_PPC64_ADDR14": 0x7,
    "R_PPC64_ADDR14_BRTAKEN": 0x8,
    "R_PPC64_ADDR14_BRNTAKEN": 0x9,
    "R_PPC64_REL24": 0xA,
    "R_PPC64_REL14": 0xB,
    "R_PPC64_REL14_BRTAKEN": 0xC,
    "R_PPC64_REL14_BRNTAKEN": 0xD,
    "R_PPC64_GOT16": 0xE,
    "R_PPC64_GOT16_LO": 0xF,
    "R_PPC64_GOT16_HI": 0x10,
    "R_PPC64_GOT16_HA": 0x11,
    "R_PPC64_COPY": 0x13,
    "R_PPC64_GLOB_DAT": 0x14,
    "R_PPC64_JMP_SLOT": 0x15,
    "R_PPC64_RELATIVE": 0x16,
    "R_PPC64_UADDR32": 0x18,
    "R_PPC64_UADDR16": 0x19,
    "R_PPC64_REL32": 0x1A,
    "R_PPC64_PLT32": 0x1B,
    "R_PPC64_PLTREL32": 0x1C,
    "R_PPC64_PLT16_LO": 0x1D,
    "R_PPC64_PLT16_HI": 0x1E,
    "R_PPC64_PLT16_HA": 0x1F,
    "R_PPC64_SECTOFF": 0x21,
    "R_PPC64_SECTOFF_LO": 0x22,
    "R_PPC64_SECTOFF_HI": 0x23,
    "R_PPC64_SECTOFF_HA": 0x24,
    "R_PPC64_ADDR30": 0x25,
    "R_PPC64_ADDR64": 0x26,
    "R_PPC64_ADDR16_HIGHER": 0x27,
    "R_PPC64_ADDR16_HIGHERA": 0x28,
    "R_PPC64_ADDR16_HIGHEST": 0x29,
    "R_PPC64_ADDR16_HIGHESTA": 0x2A,
    "R_PPC64_UADDR64": 0x2B,
    "R_PPC64_REL64": 0x2C,
    "R_PPC64_PLT64": 0x2D,
    "R_PPC64_PLTREL64": 0x2E,
    "R_PPC64_TOC16": 0x2F,
    "R_PPC64_TOC16_LO": 0x30,
    "R_PPC64_TOC16_HI": 0x31,
    "R_PPC64_TOC16_HA": 0x32,
    "R_PPC64_TOC": 0x33,
    "R_PPC64_PLTGOT16": 0x34,
    "R_PPC64_PLTGOT16_LO": 0x35,
    "R_PPC64_PLTGOT16_HI": 0x36,
    "R_PPC64_PLTGOT16_HA": 0x37,
    "R_PPC64_ADDR16_DS": 0x38,
    "R_PPC64_ADDR16_LO_DS": 0x39,
    "R_PPC64_GOT16_DS": 0x3A,
    "R_PPC64_GOT16_LO_DS": 0x3B,
    "R_PPC64_PLT16_LO_DS": 0x3C,
    "R_PPC64_SECTOFF_DS": 0x3D,
    "R_PPC64_SECTOFF_LO_DS": 0x3E,
    "R_PPC64_TOC16_DS": 0x3F,
    "R_PPC64_TOC16_LO_DS": 0x40,
    "R_PPC64_PLTGOT16_DS": 0x41,
    "R_PPC64_PLTGOT16_LO_DS": 0x42,
    "R_PPC64_TLS": 0x43,
    "R_PPC64_DTPMOD64": 0x44,
    "R_PPC64_TPREL16": 0x45,
    "R_PPC64_TPREL16_LO": 0x46,
    "R_PPC64_TPREL16_HI": 0x47,
    "R_PPC64_TPREL16_HA": 0x48,
    "R_PPC64_TPREL64": 0x49,
    "R_PPC64_DTPREL16": 0x4A,
    "R_PPC64_DTPREL16_LO": 0x4B,
    "R_PPC64_DTPREL16_HI": 0x4C,
    "R_PPC64_DTPREL16_HA": 0x4D,
    "R_PPC64_DTPREL64": 0x4E,
    "R_PPC64_GOT_TLSGD16": 0x4F,
    "R_PPC64_GOT_TLSGD16_LO": 0x50,
    "R_PPC64_GOT_TLSGD16_HI": 0x51,
    "R_PPC64_GOT_TLSGD16_HA": 0x52,
    "R_PPC64_GOT_TLSLD16": 0x53,
    "R_PPC64_GOT_TLSLD16_LO": 0x54,
    "R_PPC64_GOT_TLSLD16_HI": 0x55,
    "R_PPC64_GOT_TLSLD16_HA": 0x56,
    "R_PPC64_GOT_TPREL16_DS": 0x57,
    "R_PPC64_GOT_TPREL16_LO_DS": 0x58,
    "R_PPC64_GOT_TPREL16_HI": 0x59,
    "R_PPC64_GOT_TPREL16_HA": 0x5A,
    "R_PPC64_GOT_DTPREL16_DS": 0x5B,
    "R_PPC64_GOT_DTPREL16_LO_DS": 0x5C,
    "R_PPC64_GOT_DTPREL16_HI": 0x5D,
    "R_PPC64_GOT_DTPREL16_HA": 0x5E,
    "R_PPC64_TPREL16_DS": 0x5F,
    "R_PPC64_TPREL16_LO_DS": 0x60,
    "R_PPC64_TPREL16_HIGHER": 0x61,
    "R_PPC64_TPREL16_HIGHERA": 0x62,
    "R_PPC64_TPREL16_HIGHEST": 0x63,
    "R_PPC64_TPREL16_HIGHESTA": 0x64,
    "R_PPC64_DTPREL16_DS": 0x65,
    "R_PPC64_DTPREL16_LO_DS": 0x66,
    "R_PPC64_DTPREL16_HIGHER": 0x67,
    "R_PPC64_DTPREL16_HIGHERA": 0x68,
    "R_PPC64_DTPREL16_HIGHEST": 0x69,
    "R_PPC64_DTPREL16_HIGHESTA": 0x6A,
    "R_PPC64_TLSGD": 0x6B,
    "R_PPC64_TLSLD": 0x6C,
    "R_PPC64_TOCSAVE": 0x6D,
    "R_PPC64_ADDR16_HIGH": 0x6E,
    "R_PPC64_ADDR16_HIGHA": 0x6F,
    "R_PPC64_TPREL16_HIGH": 0x70,
    "R_PPC64_TPREL16_HIGHA": 0x71,
    "R_PPC64_DTPREL16_HIGH": 0x72,
    "R_PPC64_DTPREL16_HIGHA": 0x73,
    "R_PPC64_JMP_IREL": 0xF7,
    "R_PPC64_IRELATIVE": 0xF8,
    "R_PPC64_REL16": 0xF9,
    "R_PPC64_REL16_LO": 0xFA,
    "R_PPC64_REL16_HI": 0xFB,
    "R_PPC64_REL16_HA": 0xFC,
    "R_SH_NONE": 0x0,
    "R_SH_DIR32": 0x1,
    "R_SH_REL32": 0x2,
    "R_SH_DIR8WPN": 0x3,
    "R_SH_IND12W": 0x4,
    "R_SH_DIR8WPL": 0x5,
    "R_SH_DIR8WPZ": 0x6,
    "R_SH_DIR8BP": 0x7,
    "R_SH_DIR8W": 0x8,
    "R_SH_DIR8L": 0x9,
    "R_SH_SWITCH16": 0x19,
    "R_SH_SWITCH32": 0x1A,
    "R_SH_USES": 0x1B,
    "R_SH_COUNT": 0x1C,
    "R_SH_ALIGN": 0x1D,
    "R_SH_CODE": 0x1E,
    "R_SH_DATA": 0x1F,
    "R_SH_LABEL": 0x20,
    "R_SH_SWITCH8": 0x21,
    "R_SH_GNU_VTINHERIT": 0x22,
    "R_SH_GNU_VTENTRY": 0x23,
    "R_SH_TLS_GD_32": 0x90,
    "R_SH_TLS_LD_32": 0x91,
    "R_SH_TLS_LDO_32": 0x92,
    "R_SH_TLS_IE_32": 0x93,
    "R_SH_TLS_LE_32": 0x94,
    "R_SH_TLS_DTPMOD32": 0x95,
    "R_SH_TLS_DTPOFF32": 0x96,
    "R_SH_TLS_TPOFF32": 0x97,
    "R_SH_GOT32": 0xA0,
    "R_SH_PLT32": 0xA1,
    "R_SH_COPY": 0xA2,
    "R_SH_GLOB_DAT": 0xA3,
    "R_SH_JMP_SLOT": 0xA4,
    "R_SH_RELATIVE": 0xA5,
    "R_SH_GOTOFF": 0xA6,
    "R_SH_GOTPC": 0xA7,
    "R_SH_NUM": 0x100,
    "R_SPARC_NONE": 0x0,
    "R_SPARC_8": 0x1,
    "R_SPARC_16": 0x2,
    "R_SPARC_32": 0x3,
    "R_SPARC_DISP8": 0x4,
    "R_SPARC_DISP16": 0x5,
    "R_SPARC_DISP32": 0x6,
    "R_SPARC_WDISP30": 0x7,
    "R_SPARC_WDISP22": 0x8,
    "R_SPARC_HI22": 0x9,
    "R_SPARC_22": 0xA,
    "R_SPARC_13": 0xB,
    "R_SPARC_LO10": 0xC,
    "R_SPARC_GOT10": 0xD,
    "R_SPARC_GOT13": 0xE,
    "R_SPARC_GOT22": 0xF,
    "R_SPARC_PC10": 0x10,
    "R_SPARC_PC22": 0x11,
    "R_SPARC_WPLT30": 0x12,
    "R_SPARC_COPY": 0x13,
    "R_SPARC_GLOB_DAT": 0x14,
    "R_SPARC_JMP_SLOT": 0x15,
    "R_SPARC_RELATIVE": 0x16,
    "R_SPARC_UA32": 0x17,
    "R_SPARC_PLT32": 0x18,
    "R_SPARC_HIPLT22": 0x19,
    "R_SPARC_LOPLT10": 0x1A,
    "R_SPARC_PCPLT32": 0x1B,
    "R_SPARC_PCPLT22": 0x1C,
    "R_SPARC_PCPLT10": 0x1D,
    "R_SPARC_10": 0x1E,
    "R_SPARC_11": 0x1F,
    "R_SPARC_64": 0x20,
    "R_SPARC_OLO10": 0x21,
    "R_SPARC_HH22": 0x22,
    "R_SPARC_HM10": 0x23,
    "R_SPARC_LM22": 0x24,
    "R_SPARC_PC_HH22": 0x25,
    "R_SPARC_PC_HM10": 0x26,
    "R_SPARC_PC_LM22": 0x27,
    "R_SPARC_WDISP16": 0x28,
    "R_SPARC_WDISP19": 0x29,
    "R_SPARC_GLOB_JMP": 0x2A,
    "R_SPARC_7": 0x2B,
    "R_SPARC_5": 0x2C,
    "R_SPARC_6": 0x2D,
    "R_SPARC_DISP64": 0x2E,
    "R_SPARC_PLT64": 0x2F,
    "R_SPARC_HIX22": 0x30,
    "R_SPARC_LOX10": 0x31,
    "R_SPARC_H44": 0x32,
    "R_SPARC_M44": 0x33,
    "R_SPARC_L44": 0x34,
    "R_SPARC_REGISTER": 0x35,
    "R_SPARC_UA64": 0x36,
    "R_SPARC_UA16": 0x37,
    "R_SPARC_TLS_GD_HI22": 0x38,
    "R_SPARC_TLS_GD_LO10": 0x39,
    "R_SPARC_TLS_GD_ADD": 0x3A,
    "R_SPARC_TLS_GD_CALL": 0x3B,
    "R_SPARC_TLS_LDM_HI22": 0x3C,
    "R_SPARC_TLS_LDM_LO10": 0x3D,
    "R_SPARC_TLS_LDM_ADD": 0x3E,
    "R_SPARC_TLS_LDM_CALL": 0x3F,
    "R_SPARC_TLS_LDO_HIX22": 0x40,
    "R_SPARC_TLS_LDO_LOX10": 0x41,
    "R_SPARC_TLS_LDO_ADD": 0x42,
    "R_SPARC_TLS_IE_HI22": 0x43,
    "R_SPARC_TLS_IE_LO10": 0x44,
    "R_SPARC_TLS_IE_LD": 0x45,
    "R_SPARC_TLS_IE_LDX": 0x46,
    "R_SPARC_TLS_IE_ADD": 0x47,
    "R_SPARC_TLS_LE_HIX22": 0x48,
    "R_SPARC_TLS_LE_LOX10": 0x49,
    "R_SPARC_TLS_DTPMOD32": 0x4A,
    "R_SPARC_TLS_DTPMOD64": 0x4B,
    "R_SPARC_TLS_DTPOFF32": 0x4C,
    "R_SPARC_TLS_DTPOFF64": 0x4D,
    "R_SPARC_TLS_TPOFF32": 0x4E,
    "R_SPARC_TLS_TPOFF64": 0x4F,
    "R_SPARC_GOTDATA_HIX22": 0x50,
    "R_SPARC_GOTDATA_LOX10": 0x51,
    "R_SPARC_GOTDATA_OP_HIX22": 0x52,
    "R_SPARC_GOTDATA_OP_LOX10": 0x53,
    "R_SPARC_GOTDATA_OP": 0x54,
    "R_SPARC_H34": 0x55,
    "R_SPARC_SIZE32": 0x56,
    "R_SPARC_SIZE64": 0x57,
    "R_SPARC_WDISP10": 0x58,
    "R_SPARC_JMP_IREL": 0xF8,
    "R_SPARC_IRELATIVE": 0xF9,
    "R_SPARC_GNU_VTINHERIT": 0xFA,
    "R_SPARC_GNU_VTENTRY": 0xFB,
    "R_SPARC_REV32": 0xFC,
    "R_SPARC_NUM": 0xFD,
    "R_TILEGX_NONE": 0x0,
    "R_TILEGX_64": 0x1,
    "R_TILEGX_32": 0x2,
    "R_TILEGX_16": 0x3,
    "R_TILEGX_8": 0x4,
    "R_TILEGX_64_PCREL": 0x5,
    "R_TILEGX_32_PCREL": 0x6,
    "R_TILEGX_16_PCREL": 0x7,
    "R_TILEGX_8_PCREL": 0x8,
    "R_TILEGX_HW0": 0x9,
    "R_TILEGX_HW1": 0xA,
    "R_TILEGX_HW2": 0xB,
    "R_TILEGX_HW3": 0xC,
    "R_TILEGX_HW0_LAST": 0xD,
    "R_TILEGX_HW1_LAST": 0xE,
    "R_TILEGX_HW2_LAST": 0xF,
    "R_TILEGX_COPY": 0x10,
    "R_TILEGX_GLOB_DAT": 0x11,
    "R_TILEGX_JMP_SLOT": 0x12,
    "R_TILEGX_RELATIVE": 0x13,
    "R_TILEGX_BROFF_X1": 0x14,
    "R_TILEGX_JUMPOFF_X1": 0x15,
    "R_TILEGX_JUMPOFF_X1_PLT": 0x16,
    "R_TILEGX_IMM8_X0": 0x17,
    "R_TILEGX_IMM8_Y0": 0x18,
    "R_TILEGX_IMM8_X1": 0x19,
    "R_TILEGX_IMM8_Y1": 0x1A,
    "R_TILEGX_DEST_IMM8_X1": 0x1B,
    "R_TILEGX_MT_IMM14_X1": 0x1C,
    "R_TILEGX_MF_IMM14_X1": 0x1D,
    "R_TILEGX_MMSTART_X0": 0x1E,
    "R_TILEGX_MMEND_X0": 0x1F,
    "R_TILEGX_SHAMT_X0": 0x20,
    "R_TILEGX_SHAMT_X1": 0x21,
    "R_TILEGX_SHAMT_Y0": 0x22,
    "R_TILEGX_SHAMT_Y1": 0x23,
    "R_TILEGX_IMM16_X0_HW0": 0x24,
    "R_TILEGX_IMM16_X1_HW0": 0x25,
    "R_TILEGX_IMM16_X0_HW1": 0x26,
    "R_TILEGX_IMM16_X1_HW1": 0x27,
    "R_TILEGX_IMM16_X0_HW2": 0x28,
    "R_TILEGX_IMM16_X1_HW2": 0x29,
    "R_TILEGX_IMM16_X0_HW3": 0x2A,
    "R_TILEGX_IMM16_X1_HW3": 0x2B,
    "R_TILEGX_IMM16_X0_HW0_LAST": 0x2C,
    "R_TILEGX_IMM16_X1_HW0_LAST": 0x2D,
    "R_TILEGX_IMM16_X0_HW1_LAST": 0x2E,
    "R_TILEGX_IMM16_X1_HW1_LAST": 0x2F,
    "R_TILEGX_IMM16_X0_HW2_LAST": 0x30,
    "R_TILEGX_IMM16_X1_HW2_LAST": 0x31,
    "R_TILEGX_IMM16_X0_HW0_PCREL": 0x32,
    "R_TILEGX_IMM16_X1_HW0_PCREL": 0x33,
    "R_TILEGX_IMM16_X0_HW1_PCREL": 0x34,
    "R_TILEGX_IMM16_X1_HW1_PCREL": 0x35,
    "R_TILEGX_IMM16_X0_HW2_PCREL": 0x36,
    "R_TILEGX_IMM16_X1_HW2_PCREL": 0x37,
    "R_TILEGX_IMM16_X0_HW3_PCREL": 0x38,
    "R_TILEGX_IMM16_X1_HW3_PCREL": 0x39,
    "R_TILEGX_IMM16_X0_HW0_LAST_PCREL": 0x3A,
    "R_TILEGX_IMM16_X1_HW0_LAST_PCREL": 0x3B,
    "R_TILEGX_IMM16_X0_HW1_LAST_PCREL": 0x3C,
    "R_TILEGX_IMM16_X1_HW1_LAST_PCREL": 0x3D,
    "R_TILEGX_IMM16_X0_HW2_LAST_PCREL": 0x3E,
    "R_TILEGX_IMM16_X1_HW2_LAST_PCREL": 0x3F,
    "R_TILEGX_IMM16_X0_HW0_GOT": 0x40,
    "R_TILEGX_IMM16_X1_HW0_GOT": 0x41,
    "R_TILEGX_IMM16_X0_HW0_PLT_PCREL": 0x42,
    "R_TILEGX_IMM16_X1_HW0_PLT_PCREL": 0x43,
    "R_TILEGX_IMM16_X0_HW1_PLT_PCREL": 0x44,
    "R_TILEGX_IMM16_X1_HW1_PLT_PCREL": 0x45,
    "R_TILEGX_IMM16_X0_HW2_PLT_PCREL": 0x46,
    "R_TILEGX_IMM16_X1_HW2_PLT_PCREL": 0x47,
    "R_TILEGX_IMM16_X0_HW0_LAST_GOT": 0x48,
    "R_TILEGX_IMM16_X1_HW0_LAST_GOT": 0x49,
    "R_TILEGX_IMM16_X0_HW1_LAST_GOT": 0x4A,
    "R_TILEGX_IMM16_X1_HW1_LAST_GOT": 0x4B,
    "R_TILEGX_IMM16_X0_HW3_PLT_PCREL": 0x4C,
    "R_TILEGX_IMM16_X1_HW3_PLT_PCREL": 0x4D,
    "R_TILEGX_IMM16_X0_HW0_TLS_GD": 0x4E,
    "R_TILEGX_IMM16_X1_HW0_TLS_GD": 0x4F,
    "R_TILEGX_IMM16_X0_HW0_TLS_LE": 0x50,
    "R_TILEGX_IMM16_X1_HW0_TLS_LE": 0x51,
    "R_TILEGX_IMM16_X0_HW0_LAST_TLS_LE": 0x52,
    "R_TILEGX_IMM16_X1_HW0_LAST_TLS_LE": 0x53,
    "R_TILEGX_IMM16_X0_HW1_LAST_TLS_LE": 0x54,
    "R_TILEGX_IMM16_X1_HW1_LAST_TLS_LE": 0x55,
    "R_TILEGX_IMM16_X0_HW0_LAST_TLS_GD": 0x56,
    "R_TILEGX_IMM16_X1_HW0_LAST_TLS_GD": 0x57,
    "R_TILEGX_IMM16_X0_HW1_LAST_TLS_GD": 0x58,
    "R_TILEGX_IMM16_X1_HW1_LAST_TLS_GD": 0x59,
    "R_TILEGX_IMM16_X0_HW0_TLS_IE": 0x5C,
    "R_TILEGX_IMM16_X1_HW0_TLS_IE": 0x5D,
    "R_TILEGX_IMM16_X0_HW0_LAST_PLT_PCREL": 0x5E,
    "R_TILEGX_IMM16_X1_HW0_LAST_PLT_PCREL": 0x5F,
    "R_TILEGX_IMM16_X0_HW1_LAST_PLT_PCREL": 0x60,
    "R_TILEGX_IMM16_X1_HW1_LAST_PLT_PCREL": 0x61,
    "R_TILEGX_IMM16_X0_HW2_LAST_PLT_PCREL": 0x62,
    "R_TILEGX_IMM16_X1_HW2_LAST_PLT_PCREL": 0x63,
    "R_TILEGX_IMM16_X0_HW0_LAST_TLS_IE": 0x64,
    "R_TILEGX_IMM16_X1_HW0_LAST_TLS_IE": 0x65,
    "R_TILEGX_IMM16_X0_HW1_LAST_TLS_IE": 0x66,
    "R_TILEGX_IMM16_X1_HW1_LAST_TLS_IE": 0x67,
    "R_TILEGX_TLS_DTPMOD64": 0x6A,
    "R_TILEGX_TLS_DTPOFF64": 0x6B,
    "R_TILEGX_TLS_TPOFF64": 0x6C,
    "R_TILEGX_TLS_DTPMOD32": 0x6D,
    "R_TILEGX_TLS_DTPOFF32": 0x6E,
    "R_TILEGX_TLS_TPOFF32": 0x6F,
    "R_TILEGX_TLS_GD_CALL": 0x70,
    "R_TILEGX_IMM8_X0_TLS_GD_ADD": 0x71,
    "R_TILEGX_IMM8_X1_TLS_GD_ADD": 0x72,
    "R_TILEGX_IMM8_Y0_TLS_GD_ADD": 0x73,
    "R_TILEGX_IMM8_Y1_TLS_GD_ADD": 0x74,
    "R_TILEGX_TLS_IE_LOAD": 0x75,
    "R_TILEGX_IMM8_X0_TLS_ADD": 0x76,
    "R_TILEGX_IMM8_X1_TLS_ADD": 0x77,
    "R_TILEGX_IMM8_Y0_TLS_ADD": 0x78,
    "R_TILEGX_IMM8_Y1_TLS_ADD": 0x79,
    "R_TILEGX_GNU_VTINHERIT": 0x80,
    "R_TILEGX_GNU_VTENTRY": 0x81,
    "R_TILEGX_NUM": 0x82,
    "R_TILEPRO_NONE": 0x0,
    "R_TILEPRO_32": 0x1,
    "R_TILEPRO_16": 0x2,
    "R_TILEPRO_8": 0x3,
    "R_TILEPRO_32_PCREL": 0x4,
    "R_TILEPRO_16_PCREL": 0x5,
    "R_TILEPRO_8_PCREL": 0x6,
    "R_TILEPRO_LO16": 0x7,
    "R_TILEPRO_HI16": 0x8,
    "R_TILEPRO_HA16": 0x9,
    "R_TILEPRO_COPY": 0xA,
    "R_TILEPRO_GLOB_DAT": 0xB,
    "R_TILEPRO_JMP_SLOT": 0xC,
    "R_TILEPRO_RELATIVE": 0xD,
    "R_TILEPRO_BROFF_X1": 0xE,
    "R_TILEPRO_JOFFLONG_X1": 0xF,
    "R_TILEPRO_JOFFLONG_X1_PLT": 0x10,
    "R_TILEPRO_IMM8_X0": 0x11,
    "R_TILEPRO_IMM8_Y0": 0x12,
    "R_TILEPRO_IMM8_X1": 0x13,
    "R_TILEPRO_IMM8_Y1": 0x14,
    "R_TILEPRO_MT_IMM15_X1": 0x15,
    "R_TILEPRO_MF_IMM15_X1": 0x16,
    "R_TILEPRO_IMM16_X0": 0x17,
    "R_TILEPRO_IMM16_X1": 0x18,
    "R_TILEPRO_IMM16_X0_LO": 0x19,
    "R_TILEPRO_IMM16_X1_LO": 0x1A,
    "R_TILEPRO_IMM16_X0_HI": 0x1B,
    "R_TILEPRO_IMM16_X1_HI": 0x1C,
    "R_TILEPRO_IMM16_X0_HA": 0x1D,
    "R_TILEPRO_IMM16_X1_HA": 0x1E,
    "R_TILEPRO_IMM16_X0_PCREL": 0x1F,
    "R_TILEPRO_IMM16_X1_PCREL": 0x20,
    "R_TILEPRO_IMM16_X0_LO_PCREL": 0x21,
    "R_TILEPRO_IMM16_X1_LO_PCREL": 0x22,
    "R_TILEPRO_IMM16_X0_HI_PCREL": 0x23,
    "R_TILEPRO_IMM16_X1_HI_PCREL": 0x24,
    "R_TILEPRO_IMM16_X0_HA_PCREL": 0x25,
    "R_TILEPRO_IMM16_X1_HA_PCREL": 0x26,
    "R_TILEPRO_IMM16_X0_GOT": 0x27,
    "R_TILEPRO_IMM16_X1_GOT": 0x28,
    "R_TILEPRO_IMM16_X0_GOT_LO": 0x29,
    "R_TILEPRO_IMM16_X1_GOT_LO": 0x2A,
    "R_TILEPRO_IMM16_X0_GOT_HI": 0x2B,
    "R_TILEPRO_IMM16_X1_GOT_HI": 0x2C,
    "R_TILEPRO_IMM16_X0_GOT_HA": 0x2D,
    "R_TILEPRO_IMM16_X1_GOT_HA": 0x2E,
    "R_TILEPRO_MMSTART_X0": 0x2F,
    "R_TILEPRO_MMEND_X0": 0x30,
    "R_TILEPRO_MMSTART_X1": 0x31,
    "R_TILEPRO_MMEND_X1": 0x32,
    "R_TILEPRO_SHAMT_X0": 0x33,
    "R_TILEPRO_SHAMT_X1": 0x34,
    "R_TILEPRO_SHAMT_Y0": 0x35,
    "R_TILEPRO_SHAMT_Y1": 0x36,
    "R_TILEPRO_DEST_IMM8_X1": 0x37,
    "R_TILEPRO_TLS_GD_CALL": 0x3C,
    "R_TILEPRO_IMM8_X0_TLS_GD_ADD": 0x3D,
    "R_TILEPRO_IMM8_X1_TLS_GD_ADD": 0x3E,
    "R_TILEPRO_IMM8_Y0_TLS_GD_ADD": 0x3F,
    "R_TILEPRO_IMM8_Y1_TLS_GD_ADD": 0x40,
    "R_TILEPRO_TLS_IE_LOAD": 0x41,
    "R_TILEPRO_IMM16_X0_TLS_GD": 0x42,
    "R_TILEPRO_IMM16_X1_TLS_GD": 0x43,
    "R_TILEPRO_IMM16_X0_TLS_GD_LO": 0x44,
    "R_TILEPRO_IMM16_X1_TLS_GD_LO": 0x45,
    "R_TILEPRO_IMM16_X0_TLS_GD_HI": 0x46,
    "R_TILEPRO_IMM16_X1_TLS_GD_HI": 0x47,
    "R_TILEPRO_IMM16_X0_TLS_GD_HA": 0x48,
    "R_TILEPRO_IMM16_X1_TLS_GD_HA": 0x49,
    "R_TILEPRO_IMM16_X0_TLS_IE": 0x4A,
    "R_TILEPRO_IMM16_X1_TLS_IE": 0x4B,
    "R_TILEPRO_IMM16_X0_TLS_IE_LO": 0x4C,
    "R_TILEPRO_IMM16_X1_TLS_IE_LO": 0x4D,
    "R_TILEPRO_IMM16_X0_TLS_IE_HI": 0x4E,
    "R_TILEPRO_IMM16_X1_TLS_IE_HI": 0x4F,
    "R_TILEPRO_IMM16_X0_TLS_IE_HA": 0x50,
    "R_TILEPRO_IMM16_X1_TLS_IE_HA": 0x51,
    "R_TILEPRO_TLS_DTPMOD32": 0x52,
    "R_TILEPRO_TLS_DTPOFF32": 0x53,
    "R_TILEPRO_TLS_TPOFF32": 0x54,
    "R_TILEPRO_IMM16_X0_TLS_LE": 0x55,
    "R_TILEPRO_IMM16_X1_TLS_LE": 0x56,
    "R_TILEPRO_IMM16_X0_TLS_LE_LO": 0x57,
    "R_TILEPRO_IMM16_X1_TLS_LE_LO": 0x58,
    "R_TILEPRO_IMM16_X0_TLS_LE_HI": 0x59,
    "R_TILEPRO_IMM16_X1_TLS_LE_HI": 0x5A,
    "R_TILEPRO_IMM16_X0_TLS_LE_HA": 0x5B,
    "R_TILEPRO_IMM16_X1_TLS_LE_HA": 0x5C,
    "R_TILEPRO_GNU_VTINHERIT": 0x80,
    "R_TILEPRO_GNU_VTENTRY": 0x81,
    "R_TILEPRO_NUM": 0x82,
    "R_X86_64_NONE": 0x0,
    "R_X86_64_64": 0x1,
    "R_X86_64_PC32": 0x2,
    "R_X86_64_GOT32": 0x3,
    "R_X86_64_PLT32": 0x4,
    "R_X86_64_COPY": 0x5,
    "R_X86_64_GLOB_DAT": 0x6,
    "R_X86_64_JUMP_SLOT": 0x7,
    "R_X86_64_RELATIVE": 0x8,
    "R_X86_64_GOTPCREL": 0x9,
    "R_X86_64_32": 0xA,
    "R_X86_64_32S": 0xB,
    "R_X86_64_16": 0xC,
    "R_X86_64_PC16": 0xD,
    "R_X86_64_8": 0xE,
    "R_X86_64_PC8": 0xF,
    "R_X86_64_DTPMOD64": 0x10,
    "R_X86_64_DTPOFF64": 0x11,
    "R_X86_64_TPOFF64": 0x12,
    "R_X86_64_TLSGD": 0x13,
    "R_X86_64_TLSLD": 0x14,
    "R_X86_64_DTPOFF32": 0x15,
    "R_X86_64_GOTTPOFF": 0x16,
    "R_X86_64_TPOFF32": 0x17,
    "R_X86_64_PC64": 0x18,
    "R_X86_64_GOTOFF64": 0x19,
    "R_X86_64_GOTPC32": 0x1A,
    "R_X86_64_GOT64": 0x1B,
    "R_X86_64_GOTPCREL64": 0x1C,
    "R_X86_64_GOTPC64": 0x1D,
    "R_X86_64_GOTPLT64": 0x1E,
    "R_X86_64_PLTOFF64": 0x1F,
    "R_X86_64_SIZE32": 0x20,
    "R_X86_64_SIZE64": 0x21,
    "R_X86_64_GOTPC32_TLSDESC": 0x22,
    "R_X86_64_TLSDESC_CALL": 0x23,
    "R_X86_64_TLSDESC": 0x24,
    "R_X86_64_IRELATIVE": 0x25,
    "R_X86_64_RELATIVE64": 0x26,
    "R_X86_64_PC32_BND": 0x27,
    "R_X86_64_PLT32_BND": 0x28,
    "R_X86_64_GOTPCRELX": 0x29,
    "R_X86_64_REX_GOTPCRELX": 0x2A,
    "R_X86_64_NUM": 0x2B,
    "SHF_ALPHA_GPREL": 0x10000000,
    "SHF_ARM_ENTRYSECT": 0x10000000,
    "SHF_ARM_COMDEF": 0x80000000,
    "SHF_IA_64_SHORT": 0x10000000,
    "SHF_IA_64_NORECOV": 0x20000000,
    "SHF_MIPS_NODUPE": 0x1000000,
    "SHF_MIPS_NAMES": 0x2000000,
    "SHF_MIPS_LOCAL": 0x4000000,
    "SHF_MIPS_NOSTRIP": 0x8000000,
    "SHF_MIPS_GPREL": 0x10000000,
    "SHF_MIPS_MERGE": 0x20000000,
    "SHF_MIPS_ADDR": 0x40000000,
    "SHF_MIPS_STRINGS": 0x80000000,
    "SHF_PARISC_SHORT": 0x20000000,
    "SHF_PARISC_HUGE": 0x40000000,
    "SHF_PARISC_SBP": 0x80000000,
    "SHN_MIPS_ACOMMON": 0xFF00,
    "SHN_MIPS_TEXT": 0xFF01,
    "SHN_MIPS_DATA": 0xFF02,
    "SHN_MIPS_SCOMMON": 0xFF03,
    "SHN_MIPS_SUNDEFINED": 0xFF04,
    "SHN_PARISC_ANSI_COMMON": 0xFF00,
    "SHN_PARISC_HUGE_COMMON": 0xFF01,
    "SHT_ALPHA_DEBUG": 0x70000001,
    "SHT_ALPHA_REGINFO": 0x70000002,
    "SHT_MIPS_LIBLIST": 0x70000000,
    "SHT_MIPS_MSYM": 0x70000001,
    "SHT_MIPS_CONFLICT": 0x70000002,
    "SHT_MIPS_GPTAB": 0x70000003,
    "SHT_MIPS_UCODE": 0x70000004,
    "SHT_MIPS_DEBUG": 0x70000005,
    "SHT_MIPS_REGINFO": 0x70000006,
    "SHT_MIPS_PACKAGE": 0x70000007,
    "SHT_MIPS_PACKSYM": 0x70000008,
    "SHT_MIPS_RELD": 0x70000009,
    "SHT_MIPS_IFACE": 0x7000000B,
    "SHT_MIPS_CONTENT": 0x7000000C,
    "SHT_MIPS_OPTIONS": 0x7000000D,
    "SHT_MIPS_SHDR": 0x70000010,
    "SHT_MIPS_FDESC": 0x70000011,
    "SHT_MIPS_EXTSYM": 0x70000012,
    "SHT_MIPS_DENSE": 0x70000013,
    "SHT_MIPS_PDESC": 0x70000014,
    "SHT_MIPS_LOCSYM": 0x70000015,
    "SHT_MIPS_AUXSYM": 0x70000016,
    "SHT_MIPS_OPTSYM": 0x70000017,
    "SHT_MIPS_LOCSTR": 0x70000018,
    "SHT_MIPS_LINE": 0x70000019,
    "SHT_MIPS_RFDESC": 0x7000001A,
    "SHT_MIPS_DELTASYM": 0x7000001B,
    "SHT_MIPS_DELTAINST": 0x7000001C,
    "SHT_MIPS_DELTACLASS": 0x7000001D,
    "SHT_MIPS_DWARF": 0x7000001E,
    "SHT_MIPS_DELTADECL": 0x7000001F,
    "SHT_MIPS_SYMBOL_LIB": 0x70000020,
    "SHT_MIPS_EVENTS": 0x70000021,
    "SHT_MIPS_TRANSLATE": 0x70000022,
    "SHT_MIPS_PIXIE": 0x70000023,
    "SHT_MIPS_XLATE": 0x70000024,
    "SHT_MIPS_XLATE_DEBUG": 0x70000025,
    "SHT_MIPS_WHIRL": 0x70000026,
    "SHT_MIPS_EH_REGION": 0x70000027,
    "SHT_MIPS_XLATE_OLD": 0x70000028,
    "SHT_MIPS_PDR_EXCEPTION": 0x70000029,
    "SHT_PARISC_EXT": 0x70000000,
    "SHT_PARISC_UNWIND": 0x70000001,
    "SHT_PARISC_DOC": 0x70000002,
    "STB_MIPS_SPLIT_COMMON": 0xD,
    "STO_ALPHA_NOPV": 0x80,
    "STO_ALPHA_STD_GPLOAD": 0x88,
    "STO_MIPS_DEFAULT": 0x0,
    "STO_MIPS_INTERNAL": 0x1,
    "STO_MIPS_HIDDEN": 0x2,
    "STO_MIPS_PROTECTED": 0x3,
    "STO_MIPS_PLT": 0x8,
    "STO_MIPS_SC_ALIGN_UNUSED": 0xFF,
    "STO_PPC64_LOCAL_BIT": 0x5,
    "STT_PARISC_MILLICODE": 0xD,
    "STT_SPARC_REGISTER": 0xD,
    "IMAGE_REL_BASED_ABSOLUTE": 0,
    "IMAGE_REL_BASED_HIGH": 1,
    "IMAGE_REL_BASED_LOW": 2,
    "IMAGE_REL_BASED_HIGHLOW": 3,
    "IMAGE_REL_BASED_HIGHADJ": 4,
    "IMAGE_REL_BASED_MIPS_JMPADDR": 5,
    "IMAGE_REL_BASED_ARM_MOV32": 5,
    "IMAGE_REL_BASED_RISCV_HIGH20": 5,
    "IMAGE_REL_BASED_THUMB_MOV32": 7,
    "IMAGE_REL_BASED_RISCV_LOW12I": 7,
    "IMAGE_REL_BASED_RISCV_LOW12S": 8,
    "IMAGE_REL_BASED_MIPS_JMPADDR16": 9,
    "IMAGE_REL_BASED_DIR64": 10,
    "IMAGE_REL_AMD64_ABSOLUTE": 0x0000,
    "IMAGE_REL_AMD64_ADDR64": 0x0001,
    "IMAGE_REL_AMD64_ADDR32": 0x0002,
    "IMAGE_REL_AMD64_ADDR32NB": 0x0003,
    "IMAGE_REL_AMD64_REL32": 0x0004,
    "IMAGE_REL_AMD64_REL32_1": 0x0005,
    "IMAGE_REL_AMD64_REL32_2": 0x0006,
    "IMAGE_REL_AMD64_REL32_3": 0x0007,
    "IMAGE_REL_AMD64_REL32_4": 0x0008,
    "IMAGE_REL_AMD64_REL32_5": 0x0009,
    "IMAGE_REL_AMD64_SECTION": 0x000A,
    "IMAGE_REL_AMD64_SECREL": 0x000B,
    "IMAGE_REL_AMD64_SECREL7": 0x000C,
    "IMAGE_REL_AMD64_TOKEN": 0x000D,
    "IMAGE_REL_AMD64_SREL32": 0x000E,
    "IMAGE_REL_AMD64_PAIR": 0x000F,
    "IMAGE_REL_AMD64_SSPAN32": 0x0010,
    "IMAGE_REL_ARM_ABSOLUTE": 0x0000,
    "IMAGE_REL_ARM_ADDR32": 0x0001,
    "IMAGE_REL_ARM_ADDR32NB": 0x0002,
    "IMAGE_REL_ARM_BRANCH24": 0x0003,
    "IMAGE_REL_ARM_BRANCH11": 0x0004,
    "IMAGE_REL_ARM_SECTION": 0x000E,
    "IMAGE_REL_ARM_SECREL": 0x000F,
    "IMAGE_REL_ARM_MOV32": 0x0010,
    "IMAGE_REL_THUMB_MOV32": 0x0011,
    "IMAGE_REL_THUMB_BRANCH20": 0x0012,
    "IMAGE_REL_THUMB_BRANCH24": 0x0014,
    "IMAGE_REL_THUMB_BLX23": 0x0015,
    "IMAGE_REL_ARM_PAIR": 0x0016,
    "IMAGE_REL_ARM64_ABSOLUTE": 0x0000,
    "IMAGE_REL_ARM64_ADDR32": 0x0001,
    "IMAGE_REL_ARM64_ADDR32NB": 0x0002,
    "IMAGE_REL_ARM64_BRANCH26": 0x0003,
    "IMAGE_REL_ARM64_PAGEBASE_REL21": 0x0004,
    "IMAGE_REL_ARM64_REL21": 0x0005,
    "IMAGE_REL_ARM64_PAGEOFFSET_12A": 0x0006,
    "IMAGE_REL_ARM64_PAGEOFFSET_12L": 0x0007,
    "IMAGE_REL_ARM64_SECREL": 0x0008,
    "IMAGE_REL_ARM64_SECREL_LOW12A": 0x0009,
    "IMAGE_REL_ARM64_SECREL_HIGH12A": 0x000A,
    "IMAGE_REL_ARM64_SECREL_LOW12L": 0x000B,
    "IMAGE_REL_ARM64_TOKEN": 0x000C,
    "IMAGE_REL_ARM64_SECTION": 0x000D,
    "IMAGE_REL_ARM64_ADDR64": 0x000E,
    "IMAGE_REL_ARM64_BRANCH19": 0x000F,
    "IMAGE_REL_ARM64_BRANCH14": 0x0010,
    "IMAGE_REL_SH3_ABSOLUTE": 0x0000,
    "IMAGE_REL_SH3_DIRECT16": 0x0001,
    "IMAGE_REL_SH3_DIRECT32": 0x0002,
    "IMAGE_REL_SH3_DIRECT8": 0x0003,
    "IMAGE_REL_SH3_DIRECT8_WORD": 0x0004,
    "IMAGE_REL_SH3_DIRECT8_LONG": 0x0005,
    "IMAGE_REL_SH3_DIRECT4": 0x0006,
    "IMAGE_REL_SH3_DIRECT4_WORD": 0x0007,
    "IMAGE_REL_SH3_DIRECT4_LONG": 0x0008,
    "IMAGE_REL_SH3_PCREL8_WORD": 0x0009,
    "IMAGE_REL_SH3_PCREL8_LONG": 0x000A,
    "IMAGE_REL_SH3_PCREL12_WORD": 0x000B,
    "IMAGE_REL_SH3_STARTOF_SECTION": 0x000C,
    "IMAGE_REL_SH3_SIZEOF_SECTION": 0x000D,
    "IMAGE_REL_SH3_SECTION": 0x000E,
    "IMAGE_REL_SH3_SECREL": 0x000F,
    "IMAGE_REL_SH3_DIRECT32_NB": 0x0010,
    "IMAGE_REL_SH3_GPREL4_LONG": 0x0011,
    "IMAGE_REL_SH3_TOKEN": 0x0012,
    "IMAGE_REL_SHM_PCRELPT": 0x0013,
    "IMAGE_REL_SHM_REFLO": 0x0014,
    "IMAGE_REL_SHM_REFHALF": 0x0015,
    "IMAGE_REL_SHM_RELLO": 0x0016,
    "IMAGE_REL_SHM_RELHALF": 0x0017,
    "IMAGE_REL_SHM_PAIR": 0x0018,
    "IMAGE_REL_SHM_NOMODE": 0x8000,
    "IMAGE_REL_PPC_ABSOLUTE": 0x0000,
    "IMAGE_REL_PPC_ADDR64": 0x0001,
    "IMAGE_REL_PPC_ADDR32": 0x0002,
    "IMAGE_REL_PPC_ADDR24": 0x0003,
    "IMAGE_REL_PPC_ADDR16": 0x0004,
    "IMAGE_REL_PPC_ADDR14": 0x0005,
    "IMAGE_REL_PPC_REL24": 0x0006,
    "IMAGE_REL_PPC_REL14": 0x0007,
    "IMAGE_REL_PPC_ADDR32NB": 0x000A,
    "IMAGE_REL_PPC_SECREL": 0x000B,
    "IMAGE_REL_PPC_SECTION": 0x000C,
    "IMAGE_REL_PPC_SECREL16": 0x000F,
    "IMAGE_REL_PPC_REFHI": 0x0010,
    "IMAGE_REL_PPC_REFLO": 0x0011,
    "IMAGE_REL_PPC_PAIR": 0x0012,
    "IMAGE_REL_PPC_SECRELLO": 0x0013,
    "IMAGE_REL_PPC_GPREL": 0x0015,
    "IMAGE_REL_PPC_TOKEN": 0x0016,
    "IMAGE_REL_I386_ABSOLUTE": 0x0000,
    "IMAGE_REL_I386_DIR16": 0x0001,
    "IMAGE_REL_I386_REL16": 0x0002,
    "IMAGE_REL_I386_DIR32": 0x0006,
    "IMAGE_REL_I386_DIR32NB": 0x0007,
    "IMAGE_REL_I386_SEG12": 0x0009,
    "IMAGE_REL_I386_SECTION": 0x000A,
    "IMAGE_REL_I386_SECREL": 0x000B,
    "IMAGE_REL_I386_TOKEN": 0x000C,
    "IMAGE_REL_I386_SECREL7": 0x000D,
    "IMAGE_REL_I386_REL32": 0x0014,
    "IMAGE_REL_IA64_ABSOLUTE": 0x0000,
    "IMAGE_REL_IA64_IMM14": 0x0001,
    "IMAGE_REL_IA64_IMM22": 0x0002,
    "IMAGE_REL_IA64_IMM64": 0x0003,
    "IMAGE_REL_IA64_DIR32": 0x0004,
    "IMAGE_REL_IA64_DIR64": 0x0005,
    "IMAGE_REL_IA64_PCREL21B": 0x0006,
    "IMAGE_REL_IA64_PCREL21M": 0x0007,
    "IMAGE_REL_IA64_PCREL21F": 0x0008,
    "IMAGE_REL_IA64_GPREL22": 0x0009,
    "IMAGE_REL_IA64_LTOFF22": 0x000A,
    "IMAGE_REL_IA64_SECTION": 0x000B,
    "IMAGE_REL_IA64_SECREL22": 0x000C,
    "IMAGE_REL_IA64_SECREL64I": 0x000D,
    "IMAGE_REL_IA64_SECREL32": 0x000E,
    "IMAGE_REL_IA64_DIR32NB": 0x0010,
    "IMAGE_REL_IA64_SREL14": 0x0011,
    "IMAGE_REL_IA64_SREL22": 0x0012,
    "IMAGE_REL_IA64_SREL32": 0x0013,
    "IMAGE_REL_IA64_UREL32": 0x0014,
    "IMAGE_REL_IA64_PCREL60X": 0x0015,
    "IMAGE_REL_IA64_PCREL60B": 0x0016,
    "IMAGE_REL_IA64_PCREL60F": 0x0017,
    "IMAGE_REL_IA64_PCREL60I": 0x0018,
    "IMAGE_REL_IA64_PCREL60M": 0x0019,
    "IMAGE_REL_IA64_IMMGPREL64": 0x001A,
    "IMAGE_REL_IA64_TOKEN": 0x001B,
    "IMAGE_REL_IA64_GPREL32": 0x001C,
    "IMAGE_REL_IA64_ADDEND": 0x001F,
}



from .arch import Arch, register_arch, Endness, Register
from .tls import TLSArchInfo

try:
    import capstone as _capstone
except ImportError:
    _capstone = None

try:
    import keystone as _keystone
except ImportError:
    _keystone = None

try:
    import unicorn as _unicorn
except ImportError:
    _unicorn = None

# FIXME: Tell fish to fix whatever he was storing in info['current_function']
# TODO: Only persist t9 in PIC programs


class ArchMIPS32(Arch):
    def __init__(self, endness=Endness.BE):
        super().__init__(endness)
        if endness == Endness.BE:
            self.function_prologs = {
                rb"\x27\xbd\xff[\x00-\xff]"  # addiu $sp, xxx
                rb"\x3c\x1c[\x00-\xff][\x00-\xff]\x9c\x27[\x00-\xff][\x00-\xff]"  # lui $gp, xxx; addiu $gp, $gp, xxxx
            }
            self.function_epilogs = {
                rb"\x8f\xbf[\x00-\xff]{2}([\x00-\xff]{4}){0,4}\x03\xe0\x00\x08"  # lw ra, off(sp); ... ; jr ra
            }
            self.qemu_name = "mips"
            self.triplet = "mips-linux-gnu"
            self.linux_name = "mips"

    bits = 32
    vex_arch = "VexArchMIPS32"
    name = "MIPS32"
    ida_processor = "mipsb"
    qemu_name = "mipsel"
    linux_name = "mipsel"  # ???
    triplet = "mipsel-linux-gnu"
    max_inst_bytes = 4
    ret_offset = 16
    syscall_num_offset = 16
    call_pushes_ret = False
    stack_change = -4
    branch_delay_slot = True
    sizeof = {"short": 16, "int": 32, "long": 32, "long long": 64}
    if _capstone:
        cs_arch = _capstone.CS_ARCH_MIPS
        cs_mode = _capstone.CS_MODE_32 + _capstone.CS_MODE_LITTLE_ENDIAN
    if _keystone:
        ks_arch = _keystone.KS_ARCH_MIPS
        ks_mode = _keystone.KS_MODE_32 + _keystone.KS_MODE_LITTLE_ENDIAN
    uc_arch = _unicorn.UC_ARCH_MIPS if _unicorn else None
    uc_mode = (_unicorn.UC_MODE_32 + _unicorn.UC_MODE_LITTLE_ENDIAN) if _unicorn else None
    uc_const = _unicorn.mips_const if _unicorn else None
    uc_prefix = "UC_MIPS_" if _unicorn else None
    function_prologs = {
        rb"[\x00-\xff]\xff\xbd\x27",  # addiu $sp, xxx
        rb"[\x00-\xff][\x00-\xff]\x1c\x3c[\x00-\xff][\x00-\xff]\x9c\x27",  # lui $gp, xxx; addiu $gp, $gp, xxxx
    }
    function_epilogs = {rb"[\x00-\xff]{2}\xbf\x8f([\x00-\xff]{4}){0,4}\x08\x00\xe0\x03"}  # lw ra, off(sp); ... ; jr ra

    ret_instruction = b"\x08\x00\xE0\x03" + b"\x25\x08\x20\x00"
    nop_instruction = b"\x00\x00\x00\x00"
    instruction_alignment = 4
    register_list = [
        Register(name="zero", size=4, alias_names=("r0",)),
        Register(name="at", size=4, alias_names=("r1",), general_purpose=True),
        Register(name="v0", size=4, alias_names=("r2",), general_purpose=True, linux_entry_value="ld_destructor"),
        Register(name="v1", size=4, alias_names=("r3",), general_purpose=True),
        Register(name="a0", size=4, alias_names=("r4",), general_purpose=True, argument=True),
        Register(name="a1", size=4, alias_names=("r5",), general_purpose=True, argument=True),
        Register(name="a2", size=4, alias_names=("r6",), general_purpose=True, argument=True),
        Register(name="a3", size=4, alias_names=("r7",), general_purpose=True, argument=True),
        Register(name="t0", size=4, alias_names=("r8",), general_purpose=True),
        Register(name="t1", size=4, alias_names=("r9",), general_purpose=True),
        Register(name="t2", size=4, alias_names=("r10",), general_purpose=True),
        Register(name="t3", size=4, alias_names=("r11",), general_purpose=True),
        Register(name="t4", size=4, alias_names=("r12",), general_purpose=True),
        Register(name="t5", size=4, alias_names=("r13",), general_purpose=True),
        Register(name="t6", size=4, alias_names=("r14",), general_purpose=True),
        Register(name="t7", size=4, alias_names=("r15",), general_purpose=True),
        Register(name="s0", size=4, alias_names=("r16",), general_purpose=True),
        Register(name="s1", size=4, alias_names=("r17",), general_purpose=True),
        Register(name="s2", size=4, alias_names=("r18",), general_purpose=True),
        Register(name="s3", size=4, alias_names=("r19",), general_purpose=True),
        Register(name="s4", size=4, alias_names=("r20",), general_purpose=True),
        Register(name="s5", size=4, alias_names=("r21",), general_purpose=True),
        Register(name="s6", size=4, alias_names=("r22",), general_purpose=True),
        Register(name="s7", size=4, alias_names=("r23",), general_purpose=True),
        Register(name="t8", size=4, alias_names=("r24",), general_purpose=True),
        Register(name="t9", size=4, alias_names=("r25",), general_purpose=True, persistent=True),
        Register(name="k0", size=4, alias_names=("r26",), general_purpose=True),
        Register(name="k1", size=4, alias_names=("r27",), general_purpose=True),
        Register(name="gp", size=4, alias_names=("r28",), persistent=True),
        Register(name="sp", size=4, alias_names=("r29",), default_value=(Arch.initial_sp, True, "global")),
        Register(name="s8", size=4, alias_names=("r30", "fp", "bp"), general_purpose=True),
        Register(
            name="ra", size=4, alias_names=("r31", "lr"), general_purpose=True, persistent=True, linux_entry_value=0
        ),
        Register(name="pc", size=4, alias_names=("ip",)),
        Register(name="hi", size=4, general_purpose=True),
        Register(name="lo", size=4, general_purpose=True),
        Register(name="f0", size=8, floating_point=True, subregisters=[("f0_lo", 0, 4)]),
        Register(name="f1", size=8, floating_point=True, subregisters=[("f1_lo", 0, 4)]),
        Register(name="f2", size=8, floating_point=True, subregisters=[("f2_lo", 0, 4)]),
        Register(name="f3", size=8, floating_point=True, subregisters=[("f3_lo", 0, 4)]),
        Register(name="f4", size=8, floating_point=True, subregisters=[("f4_lo", 0, 4)]),
        Register(name="f5", size=8, floating_point=True, subregisters=[("f5_lo", 0, 4)]),
        Register(name="f6", size=8, floating_point=True, subregisters=[("f6_lo", 0, 4)]),
        Register(name="f7", size=8, floating_point=True, subregisters=[("f7_lo", 0, 4)]),
        Register(name="f8", size=8, floating_point=True, subregisters=[("f8_lo", 0, 4)]),
        Register(name="f9", size=8, floating_point=True, subregisters=[("f9_lo", 0, 4)]),
        Register(name="f10", size=8, floating_point=True, subregisters=[("f10_lo", 0, 4)]),
        Register(name="f11", size=8, floating_point=True, subregisters=[("f11_lo", 0, 4)]),
        Register(name="f12", size=8, floating_point=True, subregisters=[("f12_lo", 0, 4)]),
        Register(name="f13", size=8, floating_point=True, subregisters=[("f13_lo", 0, 4)]),
        Register(name="f14", size=8, floating_point=True, subregisters=[("f14_lo", 0, 4)]),
        Register(name="f15", size=8, floating_point=True, subregisters=[("f15_lo", 0, 4)]),
        Register(name="f16", size=8, floating_point=True, subregisters=[("f16_lo", 0, 4)]),
        Register(name="f17", size=8, floating_point=True, subregisters=[("f17_lo", 0, 4)]),
        Register(name="f18", size=8, floating_point=True, subregisters=[("f18_lo", 0, 4)]),
        Register(name="f19", size=8, floating_point=True, subregisters=[("f19_lo", 0, 4)]),
        Register(name="f20", size=8, floating_point=True, subregisters=[("f20_lo", 0, 4)]),
        Register(name="f21", size=8, floating_point=True, subregisters=[("f21_lo", 0, 4)]),
        Register(name="f22", size=8, floating_point=True, subregisters=[("f22_lo", 0, 4)]),
        Register(name="f23", size=8, floating_point=True, subregisters=[("f23_lo", 0, 4)]),
        Register(name="f24", size=8, floating_point=True, subregisters=[("f24_lo", 0, 4)]),
        Register(name="f25", size=8, floating_point=True, subregisters=[("f25_lo", 0, 4)]),
        Register(name="f26", size=8, floating_point=True, subregisters=[("f26_lo", 0, 4)]),
        Register(name="f27", size=8, floating_point=True, subregisters=[("f27_lo", 0, 4)]),
        Register(name="f28", size=8, floating_point=True, subregisters=[("f28_lo", 0, 4)]),
        Register(name="f29", size=8, floating_point=True, subregisters=[("f29_lo", 0, 4)]),
        Register(name="f30", size=8, floating_point=True, subregisters=[("f30_lo", 0, 4)]),
        Register(name="f31", size=8, floating_point=True, subregisters=[("f31_lo", 0, 4)]),
        Register(name="fir", size=4, floating_point=True),
        Register(name="fccr", size=4, floating_point=True),
        Register(name="fexr", size=4, floating_point=True),
        Register(name="fenr", size=4, floating_point=True),
        Register(name="fcsr", size=4, floating_point=True),
        Register(name="ulr", size=4),
        Register(name="emnote", size=4, artificial=True),
        Register(name="cmstart", size=4),
        Register(name="cmlen", size=4),
        Register(name="nraddr", size=4),
        Register(name="cond", size=4),
        Register(name="dspcontrol", size=4),
        Register(name="ac0", size=8),
        Register(name="ac1", size=8),
        Register(name="ac2", size=8),
        Register(name="ac3", size=8),
        Register(name="cp0_status", size=4),
        Register(name="ip_at_syscall", size=4, artificial=True),
    ]

    # see https://github.com/radare/radare/blob/master/src/include/elf/mips.h
    dynamic_tag_translation = {
        0x70000001: "DT_MIPS_RLD_VERSION",
        0x70000002: "DT_MIPS_TIME_STAMP",
        0x70000003: "DT_MIPS_ICHECKSUM",
        0x70000004: "DT_MIPS_IVERSION",
        0x70000005: "DT_MIPS_FLAGS",
        0x70000006: "DT_MIPS_BASE_ADDRESS",
        0x70000007: "DT_MIPS_MSYM",
        0x70000008: "DT_MIPS_CONFLICT",
        0x70000009: "DT_MIPS_LIBLIST",
        0x7000000A: "DT_MIPS_LOCAL_GOTNO",
        0x7000000B: "DT_MIPS_CONFLICTNO",
        0x70000010: "DT_MIPS_LIBLISTNO",
        0x70000011: "DT_MIPS_SYMTABNO",
        0x70000012: "DT_MIPS_UNREFEXTNO",
        0x70000013: "DT_MIPS_GOTSYM",
        0x70000014: "DT_MIPS_HIPAGENO",
        0x70000016: "DT_MIPS_RLD_MAP",
        0x70000017: "DT_MIPS_DELTA_CLASS",
        0x70000018: "DT_MIPS_DELTA_CLASS_NO",
        0x70000019: "DT_MIPS_DELTA_INSTANCE",
        0x7000001A: "DT_MIPS_DELTA_INSTANCE_NO",
        0x7000001B: "DT_MIPS_DELTA_RELOC",
        0x7000001C: "DT_MIPS_DELTA_RELOC_NO",
        0x7000001D: "DT_MIPS_DELTA_SYM",
        0x7000001E: "DT_MIPS_DELTA_SYM_NO",
        0x70000020: "DT_MIPS_DELTA_CLASSSYM",
        0x70000021: "DT_MIPS_DELTA_CLASSSYM_NO",
        0x70000022: "DT_MIPS_CXX_FLAGS",
        0x70000023: "DT_MIPS_PIXIE_INIT",
        0x70000024: "DT_MIPS_SYMBOL_LIB",
        0x70000025: "DT_MIPS_LOCALPAGE_GOTIDX",
        0x70000026: "DT_MIPS_LOCAL_GOTIDX",
        0x70000027: "DT_MIPS_HIDDEN_GOTIDX",
        0x70000028: "DT_MIPS_PROTECTED_GOTIDX",
        0x70000029: "DT_MIPS_OPTIONS",
        0x7000002A: "DT_MIPS_INTERFACE",
        0x7000002B: "DT_MIPS_DYNSTR_ALIGN",
        0x7000002C: "DT_MIPS_INTERFACE_SIZE",
        0x7000002D: "DT_MIPS_RLD_TEXT_RESOLVE_ADDR",
        0x7000002E: "DT_MIPS_PERF_SUFFIX",
        0x7000002F: "DT_MIPS_COMPACT_SIZE",
        0x70000030: "DT_MIPS_GP_VALUE",
        0x70000031: "DT_MIPS_AUX_DYNAMIC",
        0x70000032: "DT_MIPS_PLTGOT",
    }
    got_section_name = ".got"
    ld_linux_name = "ld.so.1"
    elf_tls = TLSArchInfo(1, 8, [], [0], [], 0x7000, 0x8000)


register_arch([r"mipsel|mipsle"], 32, Endness.LE, ArchMIPS32)
register_arch([r".*mips.*"], 32, "any", ArchMIPS32)



from typing import NewType

RegisterOffset = NewType("RegisterOffset", int)
TmpVar = NewType("TmpVar", int)

# This causes too much issues as a NewType, sot is a simple alias instead
# This means that is still legal to pass any str where a RegisterName is expected.
# The downside is that PyCharm will show the type as `str` when displaying the signature
RegisterName = str



import logging
from typing import Union

try:
    import pypcode
except ImportError:
    pypcode = None

from .arch import Arch, Endness, Register
from .tls import TLSArchInfo
from .archerror import ArchError


log = logging.getLogger("__name__")


class ArchPcode(Arch):
    """
    archinfo interface to pypcode architectures. Provides minimal mapping for
    architectural info like register file map, endianness, bit width, etc.
    """

    def __init__(self, language: Union["pypcode.ArchLanguage", str]):
        if pypcode is None:
            raise ArchError("pypcode not installed")

        if isinstance(language, str):
            language = self._get_language_by_id(language)

        self.name = language.id
        self.pcode_arch = language.id
        self.description = language.description
        self.bits = int(language.size)
        self.endness = {"little": Endness.LE, "big": Endness.BE}[language.endian]
        self.instruction_endness = self.endness
        self.sizeof = {"short": 16, "int": 32, "long": 64, "long long": 64}
        self.elf_tls = TLSArchInfo(1, 8, [], [0], [], 0, 0)

        # Build registers list
        ctx = pypcode.Context(language)
        archinfo_regs = {rname.lower(): Register(rname.lower(), r.size, r.offset) for rname, r in ctx.registers.items()}

        # Get program counter register
        pc_offset = None
        pc_tag = language.pspec.find("programcounter")
        if pc_tag is not None:
            pc_reg = pc_tag.attrib.get("register", None)
            if pc_reg is not None:
                # FIXME: Assumes RAM space
                pc_offset = ctx.registers[pc_reg].offset
                aliases = {"pc", "ip"}
                aliases.discard(pc_reg.lower())
                for alias in aliases:
                    archinfo_regs.pop(alias, None)
                archinfo_regs[pc_reg.lower()].alias_names = tuple(aliases)

        if pc_offset is None:
            log.warning("Unknown program counter register offset?")
            pc_offset = 0x80000000

        # Get stack pointer register
        sp_offset = None
        if len(language.cspecs):

            def find_matching_cid(language, desired):
                for cid in language.cspecs:
                    if cid[0] == desired:
                        return cid
                return None

            cspec_id = (
                find_matching_cid(language, "default") or find_matching_cid(language, "gcc") or list(language.cspecs)[0]
            )
            cspec = language.cspecs[cspec_id]
            sp_tag = cspec.find("stackpointer")
            if sp_tag is not None:
                sp_reg = sp_tag.attrib.get("register", None)
                if sp_reg is not None:
                    # FIXME: Assumes RAM space
                    sp_offset = ctx.registers[sp_reg].offset
                    if sp_reg.lower() != "sp":
                        if "sp" in archinfo_regs:
                            log.warning("Unexpected SP conflict")
                            del archinfo_regs["sp"]
                        archinfo_regs[sp_reg.lower()].alias_names += ("sp",)

        if sp_offset is None:
            log.warning("Unknown stack pointer register offset?")
            sp_offset = 0x80000008

        self.instruction_alignment = 1
        self.ip_offset = pc_offset
        self.sp_offset = sp_offset
        self.bp_offset = sp_offset
        self.register_list = list(archinfo_regs.values())
        self.initial_sp = (0x8000 << (self.bits - 16)) - 1
        self.linux_name = ""  # FIXME
        self.triplet = ""  # FIXME

        super().__init__(endness=self.endness, instruction_endness=self.instruction_endness)

    @staticmethod
    def _get_language_by_id(lang_id) -> "pypcode.ArchLanguage":
        for arch in pypcode.Arch.enumerate():
            for lang in arch.languages:
                if lang.id == lang_id:
                    return lang
        raise ArchError("Language not found")
